@@ -3,27 +3,30 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #define STANDALONE
+#include <assert.h>
 #include <sparse/SparseMatrix.h>
 #include <sparse/general.h>
 #include <math.h>
 #include <sparse/QuadTree.h>
 #include <stdbool.h>
+#include <stddef.h>
 #include <string.h>
-#include <cgraph/agxbuf.h>
-#include <cgraph/alloc.h>
-#include <cgraph/prisize_t.h>
+#include <cgraph/list.h>
 #include <cgraph/cgraph.h>
 #include "make_map.h"
 #include <sfdpgen/stress_model.h>
 #include "country_graph_coloring.h"
 #include <sparse/colorutil.h>
 #include <neatogen/delaunay.h>
+#include <util/agxbuf.h>
+#include <util/alloc.h>
+#include <util/prisize_t.h>
 
 #include <edgepaint/lab.h>
 #include <edgepaint/node_distinct_coloring.h>
@@ -49,7 +52,7 @@ void map_palette_optimal_coloring(char *color_scheme, SparseMatrix A0,
 
   {double *dist = NULL;
     A = SparseMatrix_symmetrize(A0, false);
-    SparseMatrix_distance_matrix(A, 0, &dist);
+    SparseMatrix_distance_matrix(A, &dist);
     SparseMatrix_delete(A);
     A = SparseMatrix_from_dense(n, n, dist);
     free(dist);
@@ -59,7 +62,7 @@ void map_palette_optimal_coloring(char *color_scheme, SparseMatrix A0,
 
   // lightness: of the form 0,70, specifying the range of lightness of LAB
   // color. Ignored if scheme is not COLOR_LAB.
-  char *lightness = "0,100";
+  int lightness[] = {0, 100};
 
   // accuracy is the threshold given so that when finding the coloring for each
   // node, the optimal is with in "accuracy" of the true global optimal.
@@ -120,7 +123,6 @@ void improve_contiguity(int n, int dim, int *grouping, SparseMatrix poly_point_m
   double dist;
   int nbad = 0, flag;
   int maxit = 10;
-  double tol = 0.001;
 
   D = SparseMatrix_get_real_adjacency_matrix_symmetrized(graph);
 
@@ -149,7 +151,7 @@ void improve_contiguity(int n, int dim, int *grouping, SparseMatrix poly_point_m
   }
 
   if (Verbose) fprintf(stderr,"ratio (edges among discontiguous regions vs total edges)=%f\n",((double) nbad)/ia[n]);
-  stress_model(dim, D, &x, maxit, tol, &flag);
+  stress_model(dim, D, &x, maxit, &flag);
 
   assert(!flag);
 
@@ -244,62 +246,53 @@ static void plot_dot_labels(FILE *f, int n, int dim, double *x, char **labels, f
 
 }
 
-static void dot_polygon(agxbuf *sbuff, int np, double *xp, double *yp,
-                        double line_width, int fill, const char *cstring) {
+DEFINE_LIST(doubles, double)
 
-  if (np > 0){
-    if (fill >= 0){
-      agxbprint(sbuff, " c %" PRISIZE_T " -%s C %" PRISIZE_T " -%s P %d ",
-                strlen(cstring), cstring, strlen(cstring), cstring, np);
+static void dot_polygon(agxbuf *sbuff, doubles_t xp, doubles_t yp,
+                        double line_width, bool fill, const char *cstring) {
+
+  assert(doubles_size(&xp) == doubles_size(&yp));
+  if (!doubles_is_empty(&xp)){
+    if (fill) {
+      agxbprint(sbuff,
+                " c %" PRISIZE_T " -%s C %" PRISIZE_T " -%s P %" PRISIZE_T " ",
+                strlen(cstring), cstring, strlen(cstring), cstring,
+                doubles_size(&xp));
     } else {
       if (line_width > 0){
 	size_t len_swidth = (size_t)snprintf(NULL, 0, "%f", line_width);
 	agxbprint(sbuff, " c %" PRISIZE_T " -%s S %" PRISIZE_T
-	          " -setlinewidth(%f) L %d ", strlen(cstring), cstring,
-	          len_swidth + 14, line_width, np);
+	          " -setlinewidth(%f) L %" PRISIZE_T " ", strlen(cstring), cstring,
+	          len_swidth + 14, line_width, doubles_size(&xp));
       } else {
-	agxbprint(sbuff, " c %" PRISIZE_T " -%s L %d ", strlen(cstring), cstring, np);
+	agxbprint(sbuff, " c %" PRISIZE_T " -%s L %" PRISIZE_T " ", strlen(cstring),
+	          cstring, doubles_size(&xp));
       }
     }
-    for (int i = 0; i < np; i++){
-      agxbprint(sbuff, " %f %f",xp[i], yp[i]);
+    for (size_t i = 0; i < doubles_size(&xp); i++) {
+      agxbprint(sbuff, " %f %f", doubles_get(&xp, i), doubles_get(&yp, i));
     }
   }
 }
 
-static void dot_one_poly(agxbuf *sbuff, double line_width, int fill, int np,
-                         double *xp, double *yp, const char *cstring) {
-  dot_polygon(sbuff, np, xp, yp, line_width, fill, cstring);
-}
-
 static void plot_dot_polygons(agxbuf *sbuff, double line_width,
                               const char *line_color, SparseMatrix polys,
                               double *x_poly, int *polys_groups, float *r,
                               float *g, float *b, const char *opacity) {
   int i, j, *ia = polys->ia, *ja = polys->ja, *a = polys->a, npolys = polys->m, nverts = polys->n, ipoly,first;
-  int np = 0;
-  int fill = -1;
-  int use_line = (line_width >= 0);
+  const bool fill = false;
+  const bool use_line = line_width >= 0;
   
   agxbuf cstring_buffer = {0};
   agxbput(&cstring_buffer, "#aaaaaaff");
   char *cstring = agxbuse(&cstring_buffer);
 
-  size_t maxlen = 0;
-  for (i = 0; i < npolys; i++) {
-    int len = ia[i + 1] - ia[i];
-    if (len > 0 && (size_t)len > maxlen) {
-      maxlen = (size_t)len;
-    }
-  }
-
-  double *xp = gv_calloc(maxlen, sizeof(double));
-  double *yp = gv_calloc(maxlen, sizeof(double));
+  doubles_t xp = {0};
+  doubles_t yp = {0};
 
   if (Verbose) fprintf(stderr,"npolys = %d\n",npolys);
   first = abs(a[0]); ipoly = first + 1;
   for (i = 0; i < npolys; i++){
-    np = 0;
     for (j = ia[i]; j < ia[i+1]; j++){
       assert(ja[j] < nverts && ja[j] >= 0);
       (void)nverts;
@@ -310,21 +303,24 @@ static void plot_dot_polygons(agxbuf *sbuff, double line_width,
 	          &cstring_buffer, opacity);
 	  cstring = agxbuse(&cstring_buffer);
 	}
-	dot_one_poly(sbuff, line_width, fill, np, xp, yp, cstring);
-	np = 0;/* start a new polygon */
+	dot_polygon(sbuff, xp, yp, line_width, fill, cstring);
+	// start a new polygon
+	doubles_clear(&xp);
+	doubles_clear(&yp);
       } 
-      xp[np] = x_poly[2*ja[j]]; yp[np++] = x_poly[2*ja[j]+1];
+      doubles_append(&xp, x_poly[2 * ja[j]]);
+      doubles_append(&yp, x_poly[2 * ja[j] + 1]);
     }
     if (use_line) {
-      dot_one_poly(sbuff, line_width, fill, np, xp, yp, line_color);
+      dot_polygon(sbuff, xp, yp, line_width, fill, line_color);
     } else {
       /* why set fill to polys_groups[i]?*/
-      dot_one_poly(sbuff, -1, 1, np, xp, yp, cstring);
+      dot_polygon(sbuff, xp, yp, -1, true, cstring);
     }
   }
   agxbfree(&cstring_buffer);
-  free(xp);
-  free(yp);
+  doubles_free(&xp);
+  doubles_free(&yp);
 }
 
 void plot_dot_map(Agraph_t* gr, int n, int dim, double *x, SparseMatrix polys,
@@ -382,22 +378,29 @@ void plot_dot_map(Agraph_t* gr, int n, int dim, double *x, SparseMatrix polys,
   agxbfree(&sbuff);
 }
 
-static void get_tri(int n, int dim, double *x, int *nt, struct Triangle **T, SparseMatrix *E) {
-   /* always contains a self edge and is symmetric.
-     input:
-     n: number of points
-     dim: dimension, only first2D used
-     x: point j is stored x[j*dim]--x[j*dim+dim-1]
-     output: 
-     nt: number of triangles
-     T: triangles
-     E: a matrix of size nxn, if two points i > j are connected by an triangulation edge, and is neighboring two triangles
-     .  t1 and t2, then A[i,j] is both t1 and t2
-   */
+/** construct triangles
+ *
+ * Always contains a self edge and is symmetric.
+ *
+ * @param n Number of points
+ * @param dim Dimension, only first2D is used
+ * @param x Point j is stored x[j × dim] -- x[j × dim + dim - 1]
+ * @param nt [out] Number of triangles
+ * @param T [out] triangles
+ * @param E [out] A matrix of size n×n, if two points i > j are connected by an
+ *   triangulation edge, and is neighboring two triangles t1 and t2, then
+ *   A[i, j] is both t1 and t2
+ * @return 0 on success
+ */
+static int get_tri(int n, int dim, double *x, int *nt, struct Triangle **T,
+                   SparseMatrix *E) {
   int i, j, i0, i1, i2, ntri;
   SparseMatrix A, B;
 
   int* trilist = get_triangles(x, n, &ntri);
+  if (trilist == NULL) {
+    return -1;
+  }
 
   *T = gv_calloc(ntri, sizeof(struct Triangle));
 
@@ -422,6 +425,7 @@ static void get_tri(int n, int dim, double *x, int *nt, struct Triangle **T, Spa
   *nt = ntri;
 
   free(trilist);
+  return 0;
 }
 
 static SparseMatrix get_country_graph(int n, SparseMatrix A, int *groups, int GRP_RANDOM, int GRP_BBOX){
@@ -462,7 +466,7 @@ static void conn_comp(int n, SparseMatrix A, int *groups, SparseMatrix *poly_poi
   int *ia, *ja;
   int one = 1, jj, i, j;
   SparseMatrix B, BB;
-  int ncomps, *comps = NULL, *comps_ptr = NULL;
+  int ncomps, *comps = NULL;
 
   B = SparseMatrix_new(n, n, 1, MATRIX_TYPE_INTEGER, FORMAT_COORD);
   ia = A->ia;
@@ -477,7 +481,7 @@ static void conn_comp(int n, SparseMatrix A, int *groups, SparseMatrix *poly_poi
   }
   BB = SparseMatrix_from_coordinate_format(B);
 
-  SparseMatrix_weakly_connected_components(BB, &ncomps, &comps, &comps_ptr);
+  int *comps_ptr = SparseMatrix_weakly_connected_components(BB, &ncomps, &comps);
   SparseMatrix_delete(B);
   SparseMatrix_delete(BB);
   *poly_point_map = SparseMatrix_new(ncomps, n, n, MATRIX_TYPE_PATTERN, FORMAT_CSR);
@@ -489,33 +493,22 @@ static void conn_comp(int n, SparseMatrix A, int *groups, SparseMatrix *poly_poi
 
 }
 
-static void get_poly_lines(int nt, SparseMatrix graph, SparseMatrix E, int ncomps, int *comps_ptr, int *comps,
-			   int *groups, int *mask, SparseMatrix *poly_lines, int **polys_groups,
-			   int GRP_RANDOM, int GRP_BBOX){
+static void get_poly_lines(int nt, SparseMatrix E, int ncomps, int *comps_ptr,
+                           int *comps, int *groups, SparseMatrix *poly_lines,
+                           int **polys_groups, int GRP_RANDOM, int GRP_BBOX) {
   /*============================================================
 
     polygon outlines 
 
     ============================================================*/
-  int i, *tlist, nz, ipoly, ipoly2, nnt, ii, jj, t1, t2, t, cur, next, nn, j, nlink, sta;
+  int i, *tlist, nz, ipoly, nnt, ii, jj, t1, t2, t, cur, next, nn, j, nlink, sta;
   int *elist, edim = 3;/* a list tell which vertex a particular vertex is linked with during poly construction.
 		since the surface is a cycle, each can only link with 2 others, the 3rd position is used to record how many links
 	      */
-  int *ie = E->ia, *je = E->ja, *e = E->a, n = E->m;
-  int *ia = NULL, *ja = NULL;
+  int *ie = E->ia, *je = E->ja, *e = E->a;
   SparseMatrix A;
-  int *gmask = NULL;
-
-
-  graph = NULL;/* we disable checking whether a polyline cross an edge for now due to issues with labels */
-  if (graph) {
-    assert(graph->m == n);
-    gmask = gv_calloc(n, sizeof(int));
-    for (i = 0; i < n; i++) gmask[i] = -1;
-    ia = graph->ia; ja = graph->ja;
-    edim = 5;/* we also store info about whether an edge of a polygon corresponds to a real edge or not. */
-  }
 
+  int *mask = gv_calloc(nt, sizeof(int));
   for (i = 0; i < nt; i++) mask[i] = -1;
   /* loop over every point in each connected component */
   elist = gv_calloc(nt * edim, sizeof(int));
@@ -533,18 +526,11 @@ static void get_poly_lines(int nt, SparseMatrix graph, SparseMatrix E, int ncomp
     for (j = comps_ptr[i]; j < comps_ptr[i+1]; j++){
       ii = comps[j];
 
-      if (graph){
-	for (jj = ia[ii]; jj < ia[ii+1]; jj++) gmask[ja[jj]] = ii;
-      }
-
       (*polys_groups)[i] = groups[ii];/* assign the grouping of each poly */
 
       /* skip the country formed by random points */
       if (groups[ii] == GRP_RANDOM || groups[ii] == GRP_BBOX) continue;
 
-      /* always skip bounding box */
-      if (groups[ii] == GRP_BBOX) continue;
-
       for (jj = ie[ii]; jj < ie[ii+1]; jj++){
 	if (groups[je[jj]] != groups[ii] && jj < nz - 1  && je[jj] == je[jj+1]){/* an triangle edge neighboring 2 triangles and two ends not in the same groups */
 	  t1 = e[jj];
@@ -552,27 +538,11 @@ static void get_poly_lines(int nt, SparseMatrix graph, SparseMatrix E, int ncomp
 
 	  nlink = elist[t1*edim + 2]%2;
 	  elist[t1*edim + nlink] = t2;/* t1->t2*/
-	  if (graph){
-	    if (gmask[je[jj]] == ii){/* a real edge */
-	      elist[t1*edim+nlink+3] = POLY_LINE_REAL_EDGE;
-	    } else {
-	      fprintf(stderr,"not real!!!\n");
-	      elist[t1*edim+nlink+3] = POLY_LINE_NOT_REAL_EDGE;
-	    }
-	  }
 	  elist[t1*edim + 2]++;
 
 	  nlink = elist[t2*edim + 2]%2;
 	  elist[t2*edim + nlink] = t1;/* t1->t2*/
 	  elist[t2*edim + 2]++;
-	  if (graph){
-	    if (gmask[je[jj]] == ii){/* a real edge */
-	      elist[t2*edim+nlink+3] = POLY_LINE_REAL_EDGE;
-	    } else {
-	      fprintf(stderr,"not real!!!\n");
-	      elist[t2*edim+nlink+3] = POLY_LINE_NOT_REAL_EDGE;
-	    }
-	  }
 
 	  tlist[nnt++] = t1; tlist[nnt++] = t2;
 	  jj++;
@@ -580,46 +550,20 @@ static void get_poly_lines(int nt, SparseMatrix graph, SparseMatrix E, int ncomp
       }
     }/* done poly edges for this component i */
 
-
-    /* debugging*/
-#ifdef DEBUG
-    /*
-    for (j = 0; j < nnt; j++) {
-      if (elist[tlist[j]*edim + 2]%2 != 0){
-	printf("wrong: elist[%d*edim+2] = %d\n",j,elist[tlist[j]*edim + 2]%);
-	assert(0);
-      }
-    }
-    */
-#endif
-
     /* form one or more (if there is a hole) polygon outlines for this component  */
     for (j = 0; j < nnt; j++){
       t = tlist[j];
       if (mask[t] != i){
 	cur = sta = t; mask[cur] = i;
 	next = neighbor(t, 1, edim, elist);
-	nlink = 1;
-	if (graph && neighbor(cur, 3, edim, elist) == POLY_LINE_NOT_REAL_EDGE){
-	  ipoly2 = - ipoly;
-	} else {
-	  ipoly2 = ipoly;
-	}
-	SparseMatrix_coordinate_form_add_entry(*poly_lines, i, cur, &ipoly2);
+	SparseMatrix_coordinate_form_add_entry(*poly_lines, i, cur, &ipoly);
 	while (next != sta){
 	  mask[next] = i;
 	  
-	  if (graph && neighbor(cur, nlink + 3, edim, elist) == POLY_LINE_NOT_REAL_EDGE){
-	    ipoly2 = -ipoly;
-	  } else {
-	    ipoly2 = ipoly;
-	  }
-	  SparseMatrix_coordinate_form_add_entry(*poly_lines, i, next, &ipoly2);
+	  SparseMatrix_coordinate_form_add_entry(*poly_lines, i, next, &ipoly);
 
 	  nn = neighbor(next, 0, edim, elist);
-	  nlink = 0;
 	  if (nn == cur) {
-	    nlink = 1;
 	    nn = neighbor(next, 1, edim, elist);
 	  }
 	  assert(nn != cur);
@@ -628,12 +572,7 @@ static void get_poly_lines(int nt, SparseMatrix graph, SparseMatrix E, int ncomp
 	  next = nn;
 	}
 
-	if (graph && neighbor(cur, nlink + 3, edim, elist) == POLY_LINE_NOT_REAL_EDGE){
-	  ipoly2 = -ipoly;
-	} else {
-	  ipoly2 = ipoly;
-	}
-	SparseMatrix_coordinate_form_add_entry(*poly_lines, i, sta, &ipoly2);/* complete a cycle by adding starting point */
+	SparseMatrix_coordinate_form_add_entry(*poly_lines, i, sta, &ipoly);/* complete a cycle by adding starting point */
 
 	ipoly++;
       }
@@ -647,6 +586,7 @@ static void get_poly_lines(int nt, SparseMatrix graph, SparseMatrix E, int ncomp
 
   free(tlist);
   free(elist);
+  free(mask);
 }
 
 static void cycle_print(int head, int *cycle, int *edge_table){
@@ -674,11 +614,12 @@ static int same_edge(int ecur, int elast, int *edge_table){
 	  || (edge_head(ecur) == edge_tail(elast) && edge_tail(ecur) == edge_head(elast));
 }
 
-static void get_polygon_solids(int nt, SparseMatrix E, int ncomps, int *comps_ptr, int *comps,
-			       int *mask, SparseMatrix *polys){
+static void get_polygon_solids(int nt, SparseMatrix E, int ncomps,
+                               int *comps_ptr, int *comps, SparseMatrix *polys)
+{
   /*============================================================
 
-    polygon slids that will be colored
+    polygon solids that will be colored
 
     ============================================================*/
   int *edge_table;/* a table of edges of the triangle graph. If two vertex u and v are connected and are adjacent to two triangles
@@ -695,7 +636,7 @@ static void get_polygon_solids(int nt, SparseMatrix E, int ncomps, int *comps_pt
 	       cycle[e][1] gives the next edge
 	     */
   int *edge_cycle_map, NOT_ON_CYCLE = -1;/* map an edge e to its position on cycle, unless it does not exist (NOT_ON_CYCLE) */
-  int *emask;/* whether an edge is seens this iter */
+  int *emask;/* whether an edge is seen this iter */
   enum {NO_DUPLICATE = -1};
   int *elist, edim = 3;/* a list tell which edge a particular vertex is linked with when a voro cell has been visited,
 		since the surface is a cycle, each vertex can only link with 2 edges, the 3rd position is used to record how many links
@@ -709,8 +650,6 @@ static void get_polygon_solids(int nt, SparseMatrix E, int ncomps, int *comps_pt
   ne = E->nz;
   edge_table = gv_calloc(ne * 2, sizeof(int));
 
-  for (i = 0; i < n; i++) mask[i] = -1;
-
   half_edges = SparseMatrix_new(n, n, 1, MATRIX_TYPE_INTEGER, FORMAT_COORD);
 
   ne = 0;
@@ -761,7 +700,6 @@ static void get_polygon_solids(int nt, SparseMatrix E, int ncomps, int *comps_pt
     if (DEBUG_CYCLE) fprintf(stderr, "\n ============  comp %d has %d members\n",i, comps_ptr[i+1]-comps_ptr[i]);
     for (k = comps_ptr[i]; k < comps_ptr[i+1]; k++){
       ii = comps[k];
-      mask[ii] = i;
       duplicate = NO_DUPLICATE;
       if (DEBUG_CYCLE) fprintf(stderr,"member = %d has %d neighbors\n",ii, ie[ii+1]-ie[ii]);
       for (j = ie[ii]; j < ie[ii+1]; j++){
@@ -928,16 +866,17 @@ static void get_polygon_solids(int nt, SparseMatrix E, int ncomps, int *comps_pt
   free(edge_table);
 }
 
-static void get_polygons(int n, int nrandom, int dim, SparseMatrix graph, int *grouping,
-			 int nt, struct Triangle *Tp, SparseMatrix E, int *nverts, double **x_poly, 
-			 SparseMatrix *poly_lines, SparseMatrix *polys, int **polys_groups, SparseMatrix *poly_point_map, SparseMatrix *country_graph){
+static void get_polygons(int n, int nrandom, int dim, int *grouping, int nt,
+                         struct Triangle *Tp, SparseMatrix E, int *nverts,
+                         double **x_poly, SparseMatrix *poly_lines,
+                         SparseMatrix *polys, int **polys_groups,
+                         SparseMatrix *poly_point_map,
+                         SparseMatrix *country_graph) {
   int i, j;
-  int *mask;
   int *groups;
   int maxgrp;
   int *comps = NULL, *comps_ptr = NULL, ncomps;
   int GRP_RANDOM, GRP_BBOX;
-  SparseMatrix B;
 
   assert(dim == 2);
   *nverts = nt;
@@ -958,7 +897,6 @@ static void get_polygons(int n, int nrandom, int dim, SparseMatrix graph, int *g
   }
   
   /* finding connected components: vertices that are connected in the triangle graph, as well as in the same group */
-  mask = gv_calloc(MAX(n + nrandom, 2 * nt), sizeof(int));
   conn_comp(n + nrandom, E, groups, poly_point_map);
 
   ncomps = (*poly_point_map)->m;
@@ -986,27 +924,29 @@ static void get_polygons(int n, int nrandom, int dim, SparseMatrix graph, int *g
     polygon outlines 
 
     ============================================================*/
-  get_poly_lines(nt, graph, E, ncomps, comps_ptr, comps, groups, mask, poly_lines, polys_groups, GRP_RANDOM, GRP_BBOX);
+  get_poly_lines(nt, E, ncomps, comps_ptr, comps, groups, poly_lines,
+                 polys_groups, GRP_RANDOM, GRP_BBOX);
 
   /*============================================================
 
     polygon solids
 
     ============================================================*/
-  get_polygon_solids(nt, E, ncomps, comps_ptr, comps, mask, polys);
+  get_polygon_solids(nt, E, ncomps, comps_ptr, comps, polys);
 
-  B = get_country_graph(n, E, groups, GRP_RANDOM, GRP_BBOX);
-  *country_graph = B;
+  *country_graph = get_country_graph(n, E, groups, GRP_RANDOM, GRP_BBOX);
 
   free(groups);
-  free(mask);
 }
 
-static void make_map_internal(bool include_OK_points,
-		      int n, int dim, double *x0, int *grouping0, SparseMatrix graph, double bounding_box_margin, int nrandom, int nedgep, 
-		      double shore_depth_tol, int *nverts, double **x_poly, 
-		      SparseMatrix *poly_lines, SparseMatrix *polys, int **polys_groups, SparseMatrix *poly_point_map,
-		      SparseMatrix *country_graph, int highlight_cluster){
+static int make_map_internal(bool include_OK_points, int n, int dim, double *x0,
+                             int *grouping0, SparseMatrix graph,
+                             double bounding_box_margin, int nrandom,
+                             int nedgep, double shore_depth_tol, int *nverts,
+                             double **x_poly, SparseMatrix *poly_lines,
+                             SparseMatrix *polys, int **polys_groups,
+                             SparseMatrix *poly_point_map,
+                             SparseMatrix *country_graph, int highlight_cluster) {
 
 
   double xmax[2], xmin[2], area, *x = x0;
@@ -1033,8 +973,8 @@ static void make_map_internal(bool include_OK_points,
 
   for (i = 0; i < n; i++){
     for (j = 0; j < dim2; j++) {
-      xmax[j] = MAX(xmax[j], x[i*dim+j]);
-      xmin[j] = MIN(xmin[j], x[i*dim+j]);
+      xmax[j] = fmax(xmax[j], x[i*dim+j]);
+      xmin[j] = fmin(xmin[j], x[i*dim+j]);
     }
   }
   boxsize[0] = xmax[0] - xmin[0];
@@ -1096,8 +1036,6 @@ static void make_map_internal(bool include_OK_points,
       qt = QuadTree_new_from_point_list(dim, n, max_qtree_level, x);
     }
   }
-  graph = NULL;
-
 
   /* generate random points for lake/sea effect */
   if (nrandom != 0){
@@ -1109,8 +1047,8 @@ static void make_map_internal(bool include_OK_points,
 	xmin[i] -= boxsize[i]*(-bounding_box_margin);
 	xmax[i] += boxsize[i]*(-bounding_box_margin);
       } else { // auto bounding box
-	xmin[i] -= MAX(boxsize[i]*0.2, 2.*shore_depth_tol);
-	xmax[i] += MAX(boxsize[i]*0.2, 2*shore_depth_tol);
+	xmin[i] -= fmax(boxsize[i] * 0.2, 2.* shore_depth_tol);
+	xmax[i] += fmax(boxsize[i] * 0.2, 2 * shore_depth_tol);
       }
     }
     if (Verbose) {
@@ -1120,14 +1058,14 @@ static void make_map_internal(bool include_OK_points,
       else if (bbm < 0)
 	fprintf (stderr, "bounding box margin: (%.06f * %.06f)", boxsize[0], -bbm);
       else
-	fprintf (stderr, "bounding box margin: %.06f", MAX(boxsize[0]*0.2, 2*shore_depth_tol));
+	fprintf(stderr, "bounding box margin: %.06f",
+	        fmax(boxsize[0] * 0.2, 2 * shore_depth_tol));
     }
     if (nrandom < 0) {
-      double n1, n2, area2;
-      area2 = (xmax[1] - xmin[1])*(xmax[0] - xmin[0]);
-      n1 = (int) area2/(shore_depth_tol*shore_depth_tol);
-      n2 = n*((int) area2/area);
-      nrandom = MAX(n1, n2);
+      const double area2 = (xmax[1] - xmin[1]) * (xmax[0] - xmin[0]);
+      const double n1 = floor(area2 / (shore_depth_tol * shore_depth_tol));
+      const double n2 = n * floor(area2 / area);
+      nrandom = fmax(n1, n2);
     }
     srand(123);
     xran = gv_calloc((nrandom + 4) * dim2, sizeof(double));
@@ -1244,16 +1182,22 @@ static void make_map_internal(bool include_OK_points,
     }
   }
 
-  get_tri(n + nrandom, dim2, xcombined, &nt, &Tp, &E);
-  get_polygons(n, nrandom, dim2, graph, grouping, nt, Tp, E, nverts, x_poly, poly_lines, polys, polys_groups,
-	       poly_point_map, country_graph);
+  int rc = 0;
+  if (get_tri(n + nrandom, dim2, xcombined, &nt, &Tp, &E) != 0) {
+    rc = -1;
+    goto done;
+  }
+  get_polygons(n, nrandom, dim2, grouping, nt, Tp, E, nverts, x_poly,
+               poly_lines, polys, polys_groups, poly_point_map, country_graph);
 
-  free(xcombined);
   SparseMatrix_delete(E);
   free(Tp);
+done:
+  free(xcombined);
   free(xran);
   if (grouping != grouping0) free(grouping);
   if (x != x0) free(x);
+  return rc;
 }
 
 static void add_point(int *n, int igrp, double **x, int *nmax, double point[], int **groups){
@@ -1277,16 +1221,16 @@ static void get_boundingbox(int n, int dim, double *x, double *width, double *bb
   bbox[2] = bbox[3] = x[1];
   
   for (i = 0; i < n; i++){
-    bbox[0] = MIN(bbox[0], x[i*dim] - width[i*dim]);
-    bbox[1] = MAX(bbox[1], x[i*dim] + width[i*dim]);
-    bbox[2] = MIN(bbox[2], x[i*dim + 1] - width[i*dim+1]);
-    bbox[3] = MAX(bbox[3], x[i*dim + 1] + width[i*dim+1]);
+    bbox[0] = fmin(bbox[0], x[i * dim] - width[i * dim]);
+    bbox[1] = fmax(bbox[1], x[i * dim] + width[i * dim]);
+    bbox[2] = fmin(bbox[2], x[i * dim + 1] - width[i * dim + 1]);
+    bbox[3] = fmax(bbox[3], x[i * dim + 1] + width[i * dim + 1]);
   }
 }
 
-void make_map_from_rectangle_groups(bool include_OK_points,
+int make_map_from_rectangle_groups(bool include_OK_points,
 				   int n, int dim, double *x, double *sizes, 
-				   int *grouping, SparseMatrix graph0, double bounding_box_margin, int nrandom, int *nart, int nedgep, 
+				   int *grouping, SparseMatrix graph, double bounding_box_margin, int nrandom, int *nart, int nedgep, 
 				   double shore_depth_tol,
 				   int *nverts, double **x_poly, 
 				   SparseMatrix *poly_lines, SparseMatrix *polys, int **polys_groups, SparseMatrix *poly_point_map, 
@@ -1353,7 +1297,6 @@ void make_map_from_rectangle_groups(bool include_OK_points,
   double point[2];
   int nadded[2];
   double delta[2];
-  SparseMatrix graph = graph0;
   double bbox[4];
 
   if (K < 0){
@@ -1370,11 +1313,13 @@ void make_map_from_rectangle_groups(bool include_OK_points,
     fprintf(stderr, "max grouping - min grouping + 1 = %d\n",maxgp - mingp + 1); 
   }
 
+  int rc = 0;
   if (!sizes){
-    make_map_internal(include_OK_points, n, dim, x, grouping, graph, bounding_box_margin, nrandom, nedgep,
-			    shore_depth_tol, nverts, x_poly, 
-			     poly_lines, polys, polys_groups, poly_point_map, country_graph, highlight_cluster);
-    return;
+    return make_map_internal(include_OK_points, n, dim, x, grouping, graph,
+                             bounding_box_margin, nrandom, nedgep,
+                             shore_depth_tol, nverts, x_poly, poly_lines, polys,
+                             polys_groups, poly_point_map, country_graph,
+                             highlight_cluster);
   } else {
 
     /* add artificial node due to node sizes */
@@ -1403,10 +1348,9 @@ void make_map_from_rectangle_groups(bool include_OK_points,
     if (shore_depth_tol < 0) {
       shore_depth_tol = -(shore_depth_tol)*avgsz;
     } else if (shore_depth_tol == 0){
-      double area;
       get_boundingbox(n, dim, x, sizes, bbox);
-      area = (bbox[1] - bbox[0])*(bbox[3] - bbox[2]);
-      shore_depth_tol = sqrt(area/(double) n); 
+      const double area = (bbox[1] - bbox[0]) * (bbox[3] - bbox[2]);
+      shore_depth_tol = sqrt(area / n);
       if (Verbose) fprintf(stderr,"setting shore length ======%f\n",shore_depth_tol);
     } else {
     }
@@ -1477,11 +1421,13 @@ void make_map_from_rectangle_groups(bool include_OK_points,
 
     }/* done adding artificial points due to node size*/
 
-    make_map_internal(include_OK_points, N, dim, X, groups, graph, bounding_box_margin, nrandom, nedgep,
-			    shore_depth_tol, nverts, x_poly, 
-			    poly_lines, polys, polys_groups, poly_point_map, country_graph, highlight_cluster);
-    if (graph != graph0) SparseMatrix_delete(graph);
+    rc = make_map_internal(include_OK_points, N, dim, X, groups, graph,
+                           bounding_box_margin, nrandom, nedgep, 
+                           shore_depth_tol, nverts, x_poly, poly_lines, polys,
+                           polys_groups, poly_point_map, country_graph,
+                           highlight_cluster);
     free(groups);
     free(X);
   }
+  return rc;
 }
