@@ -80,17 +80,18 @@
 #include <string.h>
 #include <arith.h>
 #include <gml2gv.h>
-#include <agxbuf.h>
 #include <assert.h>
-#include <cgraph/alloc.h>
-#include <cgraph/exit.h>
-#include <cgraph/stack.h>
+#include <cgraph/list.h>
+#include <util/agxbuf.h>
+#include <util/alloc.h>
+#include <util/exit.h>
 
 static gmlgraph* G;
 static gmlnode* N;
 static gmledge* E;
 static Dt_t* L;
-static gv_stack_t liststk;
+DEFINE_LIST(dts, Dt_t *)
+static dts_t liststk;
 
 static void free_attr(void *attr);
 static char *sortToStr(unsigned short sort);
@@ -154,11 +155,11 @@ static Dtdisc_t graphDisc = {
 static void
 cleanup (void)
 {
-    while (!stack_is_empty(&liststk)) {
-	Dt_t *dt = stack_pop(&liststk);
+    while (!dts_is_empty(&liststk)) {
+	Dt_t *dt = dts_pop_back(&liststk);
 	dtclose(dt);
     }
-    stack_reset(&liststk);
+    dts_free(&liststk);
     if (L) {
 	dtclose (L);
 	L = NULL;
@@ -183,7 +184,7 @@ pushAlist (void)
     Dt_t* lp = dtopen (&attrDisc, Dtqueue);
 
     if (L) {
-	stack_push(&liststk, L);
+	dts_push_back(&liststk, L);
     }
     L = lp;
 }
@@ -193,8 +194,8 @@ popAlist (void)
 {
     Dt_t* lp = L;
 
-    if (!stack_is_empty(&liststk))
-	L = stack_pop(&liststk);
+    if (!dts_is_empty(&liststk))
+	L = dts_pop_back(&liststk);
     else
 	L = NULL;
 
@@ -292,7 +293,7 @@ setDir (char* d)
 }
 
 
-#line 296 "gmlparse.c"
+#line 297 "gmlparse.c"
 
 # ifndef YY_CAST
 #  ifdef __cplusplus
@@ -420,7 +421,7 @@ extern int gmldebug;
 #if ! defined GMLSTYPE && ! defined GMLSTYPE_IS_DECLARED
 union GMLSTYPE
 {
-#line 236 "../../cmd/tools/gmlparse.y"
+#line 237 "../../cmd/tools/gmlparse.y"
 
     int i;
     char *str;
@@ -429,7 +430,7 @@ union GMLSTYPE
     gmlattr* ap;
     Dt_t*    list;
 
-#line 433 "gmlparse.c"
+#line 434 "gmlparse.c"
 
 };
 typedef union GMLSTYPE GMLSTYPE;
@@ -807,13 +808,13 @@ static const yytype_int8 yytranslate[] =
   /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_int16 yyrline[] =
 {
-       0,   259,   259,   260,   261,   264,   267,   270,   271,   274,
-     275,   278,   279,   280,   281,   288,   289,   292,   292,   295,
-     296,   299,   300,   303,   303,   306,   307,   310,   311,   312,
-     313,   316,   316,   319,   320,   323,   324,   327,   328,   329,
-     330,   331,   332,   333,   334,   335,   336,   337,   338,   339,
-     340,   341,   342,   343,   344,   345,   346,   347,   348,   349,
-     350,   351,   352,   353
+       0,   260,   260,   261,   262,   265,   268,   271,   272,   275,
+     276,   279,   280,   281,   282,   289,   290,   293,   293,   296,
+     297,   300,   301,   304,   304,   307,   308,   311,   312,   313,
+     314,   317,   317,   320,   321,   324,   325,   328,   329,   330,
+     331,   332,   333,   334,   335,   336,   337,   338,   339,   340,
+     341,   342,   343,   344,   345,   346,   347,   348,   349,   350,
+     351,   352,   353,   354
 };
 #endif
 
@@ -1695,37 +1696,37 @@ yyreduce:
   switch (yyn)
     {
   case 2:
-#line 259 "../../cmd/tools/gmlparse.y"
+#line 260 "../../cmd/tools/gmlparse.y"
                            {gmllexeof(); if (G->parent) popG(); }
-#line 1701 "gmlparse.c"
+#line 1702 "gmlparse.c"
     break;
 
   case 3:
-#line 260 "../../cmd/tools/gmlparse.y"
+#line 261 "../../cmd/tools/gmlparse.y"
               { cleanup(); YYABORT; }
-#line 1707 "gmlparse.c"
+#line 1708 "gmlparse.c"
     break;
 
   case 5:
-#line 264 "../../cmd/tools/gmlparse.y"
+#line 265 "../../cmd/tools/gmlparse.y"
               { pushG(); }
-#line 1713 "gmlparse.c"
+#line 1714 "gmlparse.c"
     break;
 
   case 11:
-#line 278 "../../cmd/tools/gmlparse.y"
+#line 279 "../../cmd/tools/gmlparse.y"
                  { dtinsert (G->nodelist, (yyvsp[0].np)); }
-#line 1719 "gmlparse.c"
+#line 1720 "gmlparse.c"
     break;
 
   case 12:
-#line 279 "../../cmd/tools/gmlparse.y"
+#line 280 "../../cmd/tools/gmlparse.y"
                  { dtinsert (G->edgelist, (yyvsp[0].ep)); }
-#line 1725 "gmlparse.c"
+#line 1726 "gmlparse.c"
     break;
 
   case 14:
-#line 281 "../../cmd/tools/gmlparse.y"
+#line 282 "../../cmd/tools/gmlparse.y"
                              { 
 		if (setDir((yyvsp[0].str))) { 
 		    yyerror("mixed directed and undirected graphs"); 
@@ -1733,269 +1734,269 @@ yyreduce:
 		    YYABORT;
 		}
 	  }
-#line 1737 "gmlparse.c"
+#line 1738 "gmlparse.c"
     break;
 
   case 15:
-#line 288 "../../cmd/tools/gmlparse.y"
+#line 289 "../../cmd/tools/gmlparse.y"
                        { dtinsert (G->attrlist, mkAttr(gv_strdup("id"), 0, INTEGER, (yyvsp[0].str), 0)); }
-#line 1743 "gmlparse.c"
+#line 1744 "gmlparse.c"
     break;
 
   case 16:
-#line 289 "../../cmd/tools/gmlparse.y"
+#line 290 "../../cmd/tools/gmlparse.y"
                       { dtinsert (G->attrlist, (yyvsp[0].ap)); }
-#line 1749 "gmlparse.c"
+#line 1750 "gmlparse.c"
     break;
 
   case 17:
-#line 292 "../../cmd/tools/gmlparse.y"
+#line 293 "../../cmd/tools/gmlparse.y"
              { N = mkNode(); }
-#line 1755 "gmlparse.c"
+#line 1756 "gmlparse.c"
     break;
 
   case 18:
-#line 292 "../../cmd/tools/gmlparse.y"
+#line 293 "../../cmd/tools/gmlparse.y"
                                              { (yyval.np) = N; N = NULL; }
-#line 1761 "gmlparse.c"
+#line 1762 "gmlparse.c"
     break;
 
   case 21:
-#line 299 "../../cmd/tools/gmlparse.y"
+#line 300 "../../cmd/tools/gmlparse.y"
                        { N->id = (yyvsp[0].str); }
-#line 1767 "gmlparse.c"
+#line 1768 "gmlparse.c"
     break;
 
   case 22:
-#line 300 "../../cmd/tools/gmlparse.y"
+#line 301 "../../cmd/tools/gmlparse.y"
                       { dtinsert (N->attrlist, (yyvsp[0].ap)); }
-#line 1773 "gmlparse.c"
+#line 1774 "gmlparse.c"
     break;
 
   case 23:
-#line 303 "../../cmd/tools/gmlparse.y"
+#line 304 "../../cmd/tools/gmlparse.y"
              { E = mkEdge(); }
-#line 1779 "gmlparse.c"
+#line 1780 "gmlparse.c"
     break;
 
   case 24:
-#line 303 "../../cmd/tools/gmlparse.y"
+#line 304 "../../cmd/tools/gmlparse.y"
                                              { (yyval.ep) = E; E = NULL; }
-#line 1785 "gmlparse.c"
+#line 1786 "gmlparse.c"
     break;
 
   case 27:
-#line 310 "../../cmd/tools/gmlparse.y"
+#line 311 "../../cmd/tools/gmlparse.y"
                            { E->source = (yyvsp[0].str); }
-#line 1791 "gmlparse.c"
+#line 1792 "gmlparse.c"
     break;
 
   case 28:
-#line 311 "../../cmd/tools/gmlparse.y"
+#line 312 "../../cmd/tools/gmlparse.y"
                            { E->target = (yyvsp[0].str); }
-#line 1797 "gmlparse.c"
+#line 1798 "gmlparse.c"
     break;
 
   case 29:
-#line 312 "../../cmd/tools/gmlparse.y"
+#line 313 "../../cmd/tools/gmlparse.y"
                        { dtinsert (E->attrlist, mkAttr(gv_strdup("id"), 0, INTEGER, (yyvsp[0].str), 0)); }
-#line 1803 "gmlparse.c"
+#line 1804 "gmlparse.c"
     break;
 
   case 30:
-#line 313 "../../cmd/tools/gmlparse.y"
+#line 314 "../../cmd/tools/gmlparse.y"
                       { dtinsert (E->attrlist, (yyvsp[0].ap)); }
-#line 1809 "gmlparse.c"
+#line 1810 "gmlparse.c"
     break;
 
   case 31:
-#line 316 "../../cmd/tools/gmlparse.y"
+#line 317 "../../cmd/tools/gmlparse.y"
                 {pushAlist(); }
-#line 1815 "gmlparse.c"
+#line 1816 "gmlparse.c"
     break;
 
   case 32:
-#line 316 "../../cmd/tools/gmlparse.y"
+#line 317 "../../cmd/tools/gmlparse.y"
                                              { (yyval.list) = popAlist(); }
-#line 1821 "gmlparse.c"
+#line 1822 "gmlparse.c"
     break;
 
   case 35:
-#line 323 "../../cmd/tools/gmlparse.y"
+#line 324 "../../cmd/tools/gmlparse.y"
                           { dtinsert (L, (yyvsp[0].ap)); }
-#line 1827 "gmlparse.c"
+#line 1828 "gmlparse.c"
     break;
 
   case 36:
-#line 324 "../../cmd/tools/gmlparse.y"
+#line 325 "../../cmd/tools/gmlparse.y"
                     { dtinsert (L, (yyvsp[0].ap)); }
-#line 1833 "gmlparse.c"
+#line 1834 "gmlparse.c"
     break;
 
   case 37:
-#line 327 "../../cmd/tools/gmlparse.y"
+#line 328 "../../cmd/tools/gmlparse.y"
                          { (yyval.ap) = mkAttr ((yyvsp[-1].str), 0, INTEGER, (yyvsp[0].str), 0); }
-#line 1839 "gmlparse.c"
+#line 1840 "gmlparse.c"
     break;
 
   case 38:
-#line 328 "../../cmd/tools/gmlparse.y"
+#line 329 "../../cmd/tools/gmlparse.y"
                          { (yyval.ap) = mkAttr ((yyvsp[-1].str), 0, REAL, (yyvsp[0].str), 0); }
-#line 1845 "gmlparse.c"
+#line 1846 "gmlparse.c"
     break;
 
   case 39:
-#line 329 "../../cmd/tools/gmlparse.y"
+#line 330 "../../cmd/tools/gmlparse.y"
                          { (yyval.ap) = mkAttr ((yyvsp[-1].str), 0, STRING, (yyvsp[0].str), 0); }
-#line 1851 "gmlparse.c"
+#line 1852 "gmlparse.c"
     break;
 
   case 40:
-#line 330 "../../cmd/tools/gmlparse.y"
+#line 331 "../../cmd/tools/gmlparse.y"
                            { (yyval.ap) = mkAttr ((yyvsp[-1].str), 0, LIST, 0, (yyvsp[0].list)); }
-#line 1857 "gmlparse.c"
+#line 1858 "gmlparse.c"
     break;
 
   case 41:
-#line 331 "../../cmd/tools/gmlparse.y"
+#line 332 "../../cmd/tools/gmlparse.y"
                        { (yyval.ap) = mkAttr (0, XVAL, REAL, (yyvsp[0].str), 0); }
-#line 1863 "gmlparse.c"
+#line 1864 "gmlparse.c"
     break;
 
   case 42:
-#line 332 "../../cmd/tools/gmlparse.y"
+#line 333 "../../cmd/tools/gmlparse.y"
                           { (yyval.ap) = mkAttr (0, XVAL, REAL, (yyvsp[0].str), 0); }
-#line 1869 "gmlparse.c"
+#line 1870 "gmlparse.c"
     break;
 
   case 43:
-#line 333 "../../cmd/tools/gmlparse.y"
+#line 334 "../../cmd/tools/gmlparse.y"
                        { (yyval.ap) = mkAttr (0, YVAL, REAL, (yyvsp[0].str), 0); }
-#line 1875 "gmlparse.c"
+#line 1876 "gmlparse.c"
     break;
 
   case 44:
-#line 334 "../../cmd/tools/gmlparse.y"
+#line 335 "../../cmd/tools/gmlparse.y"
                        { (yyval.ap) = mkAttr (0, WVAL, REAL, (yyvsp[0].str), 0); }
-#line 1881 "gmlparse.c"
+#line 1882 "gmlparse.c"
     break;
 
   case 45:
-#line 335 "../../cmd/tools/gmlparse.y"
+#line 336 "../../cmd/tools/gmlparse.y"
                        { (yyval.ap) = mkAttr (0, HVAL, REAL, (yyvsp[0].str), 0); }
-#line 1887 "gmlparse.c"
+#line 1888 "gmlparse.c"
     break;
 
   case 46:
-#line 336 "../../cmd/tools/gmlparse.y"
+#line 337 "../../cmd/tools/gmlparse.y"
                           { (yyval.ap) = mkAttr (0, LABEL, STRING, (yyvsp[0].str), 0); }
-#line 1893 "gmlparse.c"
+#line 1894 "gmlparse.c"
     break;
 
   case 47:
-#line 337 "../../cmd/tools/gmlparse.y"
+#line 338 "../../cmd/tools/gmlparse.y"
                                { (yyval.ap) = mkAttr (0, GRAPHICS, LIST, 0, (yyvsp[0].list)); }
-#line 1899 "gmlparse.c"
+#line 1900 "gmlparse.c"
     break;
 
   case 48:
-#line 338 "../../cmd/tools/gmlparse.y"
+#line 339 "../../cmd/tools/gmlparse.y"
                                     { (yyval.ap) = mkAttr (0, LABELGRAPHICS, LIST, 0, (yyvsp[0].list)); }
-#line 1905 "gmlparse.c"
+#line 1906 "gmlparse.c"
     break;
 
   case 49:
-#line 339 "../../cmd/tools/gmlparse.y"
+#line 340 "../../cmd/tools/gmlparse.y"
                          { (yyval.ap) = mkAttr (0, TYPE, STRING, (yyvsp[0].str), 0); }
-#line 1911 "gmlparse.c"
+#line 1912 "gmlparse.c"
     break;
 
   case 50:
-#line 340 "../../cmd/tools/gmlparse.y"
+#line 341 "../../cmd/tools/gmlparse.y"
                          { (yyval.ap) = mkAttr (0, FILL, STRING, (yyvsp[0].str), 0); }
-#line 1917 "gmlparse.c"
+#line 1918 "gmlparse.c"
     break;
 
   case 51:
-#line 341 "../../cmd/tools/gmlparse.y"
+#line 342 "../../cmd/tools/gmlparse.y"
                             { (yyval.ap) = mkAttr (0, OUTLINE, STRING, (yyvsp[0].str), 0); }
-#line 1923 "gmlparse.c"
+#line 1924 "gmlparse.c"
     break;
 
   case 52:
-#line 342 "../../cmd/tools/gmlparse.y"
+#line 343 "../../cmd/tools/gmlparse.y"
                                  { (yyval.ap) = mkAttr (0, OUTLINESTYLE, STRING, (yyvsp[0].str), 0); }
-#line 1929 "gmlparse.c"
+#line 1930 "gmlparse.c"
     break;
 
   case 53:
-#line 343 "../../cmd/tools/gmlparse.y"
+#line 344 "../../cmd/tools/gmlparse.y"
                                   { (yyval.ap) = mkAttr (0, OUTLINEWIDTH, INTEGER, (yyvsp[0].str), 0); }
-#line 1935 "gmlparse.c"
+#line 1936 "gmlparse.c"
     break;
 
   case 54:
-#line 344 "../../cmd/tools/gmlparse.y"
+#line 345 "../../cmd/tools/gmlparse.y"
                         { (yyval.ap) = mkAttr (0, WIDTH, REAL, (yyvsp[0].str), 0); }
-#line 1941 "gmlparse.c"
+#line 1942 "gmlparse.c"
     break;
 
   case 55:
-#line 345 "../../cmd/tools/gmlparse.y"
+#line 346 "../../cmd/tools/gmlparse.y"
                            { (yyval.ap) = mkAttr (0, WIDTH, INTEGER, (yyvsp[0].str), 0); }
-#line 1947 "gmlparse.c"
+#line 1948 "gmlparse.c"
     break;
 
   case 56:
-#line 346 "../../cmd/tools/gmlparse.y"
+#line 347 "../../cmd/tools/gmlparse.y"
                           { (yyval.ap) = mkAttr (0, STYLE, STRING, (yyvsp[0].str), 0); }
-#line 1953 "gmlparse.c"
+#line 1954 "gmlparse.c"
     break;
 
   case 57:
-#line 347 "../../cmd/tools/gmlparse.y"
+#line 348 "../../cmd/tools/gmlparse.y"
                             { (yyval.ap) = mkAttr (0, STYLE, LIST, 0, (yyvsp[0].list)); }
-#line 1959 "gmlparse.c"
+#line 1960 "gmlparse.c"
     break;
 
   case 58:
-#line 348 "../../cmd/tools/gmlparse.y"
+#line 349 "../../cmd/tools/gmlparse.y"
                            { (yyval.ap) = mkAttr (0, LINE, LIST, 0, (yyvsp[0].list)); }
-#line 1965 "gmlparse.c"
+#line 1966 "gmlparse.c"
     break;
 
   case 59:
-#line 349 "../../cmd/tools/gmlparse.y"
+#line 350 "../../cmd/tools/gmlparse.y"
                             { (yyval.ap) = mkAttr (0, POINT, LIST, 0, (yyvsp[0].list)); }
-#line 1971 "gmlparse.c"
+#line 1972 "gmlparse.c"
     break;
 
   case 60:
-#line 350 "../../cmd/tools/gmlparse.y"
+#line 351 "../../cmd/tools/gmlparse.y"
                          { (yyval.ap) = mkAttr (0, TEXT, STRING, (yyvsp[0].str), 0); }
-#line 1977 "gmlparse.c"
+#line 1978 "gmlparse.c"
     break;
 
   case 61:
-#line 351 "../../cmd/tools/gmlparse.y"
+#line 352 "../../cmd/tools/gmlparse.y"
                              { (yyval.ap) = mkAttr (0, FONTNAME, STRING, (yyvsp[0].str), 0); }
-#line 1983 "gmlparse.c"
+#line 1984 "gmlparse.c"
     break;
 
   case 62:
-#line 352 "../../cmd/tools/gmlparse.y"
+#line 353 "../../cmd/tools/gmlparse.y"
                               { (yyval.ap) = mkAttr (0, FONTNAME, INTEGER, (yyvsp[0].str), 0); }
-#line 1989 "gmlparse.c"
+#line 1990 "gmlparse.c"
     break;
 
   case 63:
-#line 353 "../../cmd/tools/gmlparse.y"
+#line 354 "../../cmd/tools/gmlparse.y"
                           { (yyval.ap) = mkAttr (0, COLOR, STRING, (yyvsp[0].str), 0); }
-#line 1995 "gmlparse.c"
+#line 1996 "gmlparse.c"
     break;
 
 
-#line 1999 "gmlparse.c"
+#line 2000 "gmlparse.c"
 
       default: break;
     }
@@ -2227,7 +2228,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 356 "../../cmd/tools/gmlparse.y"
+#line 357 "../../cmd/tools/gmlparse.y"
 
 
 static void free_attr(void *attr) {
@@ -2544,9 +2545,6 @@ addAttrs (Agobj_t* obj, Dt_t* alist, agxbuf* xb, agxbuf* unk)
 	    else
 		unknown (obj, ap, xb);
 	}
-	else if (ap->sort == LABEL && AGTYPE(obj) != AGRAPH) {
-	    agsafeset (obj, "name", ap->u.value, "");
-	}
 	else
 	    unknown (obj, ap, xb);
     }
