@@ -74,17 +74,17 @@
 #define yychar          aagchar
 
 /* First part of user prologue.  */
-#line 23 "../../lib/cgraph/grammar.y"
+#line 37 "../../lib/cgraph/grammar.y"
 
 
 #include <stdbool.h>
 #include <stdio.h>
 #include <cghdr.h>
-#include <cgraph/agxbuf.h>
-#include <cgraph/alloc.h>
-#include <cgraph/streq.h>
-#include <cgraph/unreachable.h>
 #include <stddef.h>
+#include <util/alloc.h>
+#include <util/streq.h>
+#include <util/unreachable.h>
+
 extern void aagerror(const char*);
 
 static const char Key[] = "key";
@@ -194,6 +194,23 @@ static gstack_t *S;
 #if AAGDEBUG
 extern int aagdebug;
 #endif
+/* "%code requires" blocks.  */
+#line 22 "../../lib/cgraph/grammar.y"
+
+#include <cghdr.h>
+#include <util/agxbuf.h>
+
+struct gstack_s;
+
+struct aagextra_s {
+	int dummy; /* struct must not be empty */
+	/* Common */
+	/* Parser */
+	/* Lexer */
+};
+
+
+#line 214 "grammar.c"
 
 /* Token type.  */
 #ifndef AAGTOKENTYPE
@@ -230,13 +247,13 @@ extern int aagdebug;
 #if ! defined AAGSTYPE && ! defined AAGSTYPE_IS_DECLARED
 union AAGSTYPE
 {
-#line 93 "../../lib/cgraph/grammar.y"
+#line 107 "../../lib/cgraph/grammar.y"
 
 			int				i;
 			char			*str;
 			struct Agnode_s	*n;
 
-#line 240 "grammar.c"
+#line 257 "grammar.c"
 
 };
 typedef union AAGSTYPE AAGSTYPE;
@@ -612,12 +629,12 @@ static const yytype_int8 yytranslate[] =
   /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
-       0,   110,   110,   111,   112,   115,   117,   120,   120,   122,
-     122,   124,   124,   126,   126,   128,   128,   130,   130,   132,
-     133,   136,   140,   140,   142,   142,   142,   143,   147,   147,
-     149,   150,   151,   154,   155,   158,   159,   160,   163,   164,
-     167,   167,   169,   171,   172,   174,   177,   180,   183,   183,
-     186,   187,   188,   191,   191,   191,   193,   194,   197,   198
+       0,   124,   124,   125,   126,   129,   131,   134,   134,   136,
+     136,   138,   138,   140,   140,   142,   142,   144,   144,   146,
+     147,   150,   154,   154,   156,   156,   156,   157,   161,   161,
+     163,   164,   165,   168,   169,   172,   173,   174,   177,   178,
+     181,   181,   183,   185,   186,   188,   191,   194,   197,   197,
+     200,   201,   202,   205,   205,   205,   207,   208,   211,   212
 };
 #endif
 
@@ -1458,211 +1475,211 @@ yyreduce:
   switch (yyn)
     {
   case 2:
-#line 110 "../../lib/cgraph/grammar.y"
+#line 124 "../../lib/cgraph/grammar.y"
                             {freestack(); endgraph();}
-#line 1464 "grammar.c"
+#line 1481 "grammar.c"
     break;
 
   case 3:
-#line 111 "../../lib/cgraph/grammar.y"
+#line 125 "../../lib/cgraph/grammar.y"
                                         {if (G) {freestack(); endgraph(); agclose(G); G = Ag_G_global = NULL;}}
-#line 1470 "grammar.c"
+#line 1487 "grammar.c"
     break;
 
   case 6:
-#line 117 "../../lib/cgraph/grammar.y"
+#line 131 "../../lib/cgraph/grammar.y"
                                                                  {startgraph((yyvsp[0].str),(yyvsp[-1].i) != 0,(yyvsp[-2].i) != 0);}
-#line 1476 "grammar.c"
+#line 1493 "grammar.c"
     break;
 
   case 7:
-#line 120 "../../lib/cgraph/grammar.y"
+#line 134 "../../lib/cgraph/grammar.y"
                      {(yyval.str)=(yyvsp[0].str);}
-#line 1482 "grammar.c"
+#line 1499 "grammar.c"
     break;
 
   case 8:
-#line 120 "../../lib/cgraph/grammar.y"
+#line 134 "../../lib/cgraph/grammar.y"
                                             {(yyval.str)=0;}
-#line 1488 "grammar.c"
+#line 1505 "grammar.c"
     break;
 
   case 9:
-#line 122 "../../lib/cgraph/grammar.y"
+#line 136 "../../lib/cgraph/grammar.y"
                              {(yyval.i)=1;}
-#line 1494 "grammar.c"
+#line 1511 "grammar.c"
     break;
 
   case 10:
-#line 122 "../../lib/cgraph/grammar.y"
+#line 136 "../../lib/cgraph/grammar.y"
                                                     {(yyval.i)=0;}
-#line 1500 "grammar.c"
+#line 1517 "grammar.c"
     break;
 
   case 11:
-#line 124 "../../lib/cgraph/grammar.y"
+#line 138 "../../lib/cgraph/grammar.y"
                                 {(yyval.i) = 0;}
-#line 1506 "grammar.c"
+#line 1523 "grammar.c"
     break;
 
   case 12:
-#line 124 "../../lib/cgraph/grammar.y"
+#line 138 "../../lib/cgraph/grammar.y"
                                                           {(yyval.i) = 1;}
-#line 1512 "grammar.c"
+#line 1529 "grammar.c"
     break;
 
   case 21:
-#line 137 "../../lib/cgraph/grammar.y"
+#line 151 "../../lib/cgraph/grammar.y"
                                         {if ((yyvsp[-1].i)) endedge(); else endnode();}
-#line 1518 "grammar.c"
+#line 1535 "grammar.c"
     break;
 
   case 24:
-#line 142 "../../lib/cgraph/grammar.y"
+#line 156 "../../lib/cgraph/grammar.y"
                                  {getedgeitems();}
-#line 1524 "grammar.c"
+#line 1541 "grammar.c"
     break;
 
   case 25:
-#line 142 "../../lib/cgraph/grammar.y"
+#line 156 "../../lib/cgraph/grammar.y"
                                                           {getedgeitems();}
-#line 1530 "grammar.c"
+#line 1547 "grammar.c"
     break;
 
   case 26:
-#line 142 "../../lib/cgraph/grammar.y"
+#line 156 "../../lib/cgraph/grammar.y"
                                                                                       {(yyval.i) = 1;}
-#line 1536 "grammar.c"
+#line 1553 "grammar.c"
     break;
 
   case 27:
-#line 143 "../../lib/cgraph/grammar.y"
+#line 157 "../../lib/cgraph/grammar.y"
                                             {(yyval.i) = 0;}
-#line 1542 "grammar.c"
+#line 1559 "grammar.c"
     break;
 
   case 30:
-#line 149 "../../lib/cgraph/grammar.y"
+#line 163 "../../lib/cgraph/grammar.y"
                        {appendnode((yyvsp[0].str),NULL,NULL);}
-#line 1548 "grammar.c"
+#line 1565 "grammar.c"
     break;
 
   case 31:
-#line 150 "../../lib/cgraph/grammar.y"
+#line 164 "../../lib/cgraph/grammar.y"
                             {appendnode((yyvsp[-2].str),(yyvsp[0].str),NULL);}
-#line 1554 "grammar.c"
+#line 1571 "grammar.c"
     break;
 
   case 32:
-#line 151 "../../lib/cgraph/grammar.y"
+#line 165 "../../lib/cgraph/grammar.y"
                                      {appendnode((yyvsp[-4].str),(yyvsp[-2].str),(yyvsp[0].str));}
-#line 1560 "grammar.c"
+#line 1577 "grammar.c"
     break;
 
   case 33:
-#line 154 "../../lib/cgraph/grammar.y"
+#line 168 "../../lib/cgraph/grammar.y"
                                                   {attrstmt((yyvsp[-2].i),(yyvsp[-1].str));}
-#line 1566 "grammar.c"
+#line 1583 "grammar.c"
     break;
 
   case 34:
-#line 155 "../../lib/cgraph/grammar.y"
+#line 169 "../../lib/cgraph/grammar.y"
                                          {attrstmt(T_graph,NULL);}
-#line 1572 "grammar.c"
+#line 1589 "grammar.c"
     break;
 
   case 35:
-#line 158 "../../lib/cgraph/grammar.y"
+#line 172 "../../lib/cgraph/grammar.y"
                         {(yyval.i) = T_graph;}
-#line 1578 "grammar.c"
+#line 1595 "grammar.c"
     break;
 
   case 36:
-#line 159 "../../lib/cgraph/grammar.y"
+#line 173 "../../lib/cgraph/grammar.y"
                                  {(yyval.i) = T_node;}
-#line 1584 "grammar.c"
+#line 1601 "grammar.c"
     break;
 
   case 37:
-#line 160 "../../lib/cgraph/grammar.y"
+#line 174 "../../lib/cgraph/grammar.y"
                                  {(yyval.i) = T_edge;}
-#line 1590 "grammar.c"
+#line 1607 "grammar.c"
     break;
 
   case 38:
-#line 163 "../../lib/cgraph/grammar.y"
+#line 177 "../../lib/cgraph/grammar.y"
                         {(yyval.str) = (yyvsp[-1].str);}
-#line 1596 "grammar.c"
+#line 1613 "grammar.c"
     break;
 
   case 39:
-#line 164 "../../lib/cgraph/grammar.y"
+#line 178 "../../lib/cgraph/grammar.y"
                                       {(yyval.str) = NULL; }
-#line 1602 "grammar.c"
+#line 1619 "grammar.c"
     break;
 
   case 46:
-#line 177 "../../lib/cgraph/grammar.y"
+#line 191 "../../lib/cgraph/grammar.y"
                                  {appendattr((yyvsp[-2].str),(yyvsp[0].str));}
-#line 1608 "grammar.c"
+#line 1625 "grammar.c"
     break;
 
   case 48:
-#line 183 "../../lib/cgraph/grammar.y"
+#line 197 "../../lib/cgraph/grammar.y"
                               {opensubg((yyvsp[0].str));}
-#line 1614 "grammar.c"
+#line 1631 "grammar.c"
     break;
 
   case 49:
-#line 183 "../../lib/cgraph/grammar.y"
+#line 197 "../../lib/cgraph/grammar.y"
                                                    {closesubg();}
-#line 1620 "grammar.c"
+#line 1637 "grammar.c"
     break;
 
   case 50:
-#line 186 "../../lib/cgraph/grammar.y"
+#line 200 "../../lib/cgraph/grammar.y"
                                   {(yyval.str)=(yyvsp[0].str);}
-#line 1626 "grammar.c"
+#line 1643 "grammar.c"
     break;
 
   case 51:
-#line 187 "../../lib/cgraph/grammar.y"
+#line 201 "../../lib/cgraph/grammar.y"
                                       {(yyval.str)=NULL;}
-#line 1632 "grammar.c"
+#line 1649 "grammar.c"
     break;
 
   case 52:
-#line 188 "../../lib/cgraph/grammar.y"
+#line 202 "../../lib/cgraph/grammar.y"
                                       {(yyval.str)=NULL;}
-#line 1638 "grammar.c"
+#line 1655 "grammar.c"
     break;
 
   case 56:
-#line 193 "../../lib/cgraph/grammar.y"
+#line 207 "../../lib/cgraph/grammar.y"
                   {(yyval.str) = (yyvsp[0].str);}
-#line 1644 "grammar.c"
+#line 1661 "grammar.c"
     break;
 
   case 57:
-#line 194 "../../lib/cgraph/grammar.y"
+#line 208 "../../lib/cgraph/grammar.y"
                                  {(yyval.str) = (yyvsp[0].str);}
-#line 1650 "grammar.c"
+#line 1667 "grammar.c"
     break;
 
   case 58:
-#line 197 "../../lib/cgraph/grammar.y"
+#line 211 "../../lib/cgraph/grammar.y"
                    {(yyval.str) = (yyvsp[0].str);}
-#line 1656 "grammar.c"
+#line 1673 "grammar.c"
     break;
 
   case 59:
-#line 198 "../../lib/cgraph/grammar.y"
+#line 212 "../../lib/cgraph/grammar.y"
                                              {(yyval.str) = concat((yyvsp[-2].str),(yyvsp[0].str));}
-#line 1662 "grammar.c"
+#line 1679 "grammar.c"
     break;
 
 
-#line 1666 "grammar.c"
+#line 1683 "grammar.c"
 
       default: break;
     }
@@ -1894,13 +1911,15 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 200 "../../lib/cgraph/grammar.y"
+#line 214 "../../lib/cgraph/grammar.y"
 
 
 static item *newitem(int tag, void *p0, char *p1)
 {
 	item	*rv = agalloc(G,sizeof(item));
-	rv->tag = tag; rv->u.name = (char*)p0; rv->str = p1;
+	rv->tag = tag;
+	rv->u.name = p0;
+	rv->str = p1;
 	return rv;
 }
 
@@ -1965,7 +1984,7 @@ static void appendattr(char *name, char *value)
 
 	assert(value != NULL);
 	v = cons_attr(name,value);
-	listapp(&(S->attrlist),v);
+	listapp(&S->attrlist, v);
 }
 
 static void bindattrs(int kind)
@@ -2034,14 +2053,14 @@ static void attrstmt(int tkind, char *macroname)
 	for (aptr = S->attrlist.first; aptr; aptr = aptr->next) {
 		/* If the tag is still T_atom, aptr->u.asym has not been set */
 		if (aptr->tag == T_atom) continue;
-		if (!(aptr->u.asym->fixed) || (S->g != G))
+		if (!aptr->u.asym->fixed || S->g != G)
 			sym = agattr(S->g,kind,aptr->u.asym->name,aptr->str);
 		else
 			sym = aptr->u.asym;
 		if (S->g == G)
 			sym->print = true;
 	}
-	deletelist(&(S->attrlist));
+	deletelist(&S->attrlist);
 }
 
 /* nodes */
@@ -2054,7 +2073,7 @@ static void appendnode(char *name, char *port, char *sport)
 		port = concatPort (port, sport);
 	}
 	elt = cons_node(agnode(S->g, name, 1), port);
-	listapp(&(S->nodelist),elt);
+	listapp(&S->nodelist, elt);
 	agstrfree(G,name);
 }
 
@@ -2070,9 +2089,9 @@ static void endnode(void)
 	bindattrs(AGNODE);
 	for (ptr = S->nodelist.first; ptr; ptr = ptr->next)
 		applyattrs(ptr->u.n);
-	deletelist(&(S->nodelist));
-	deletelist(&(S->attrlist));
-	deletelist(&(S->edgelist));
+	deletelist(&S->nodelist);
+	deletelist(&S->attrlist);
+	deletelist(&S->edgelist);
 	S->subg = 0;  /* notice a pattern here? :-( */
 }
 
@@ -2088,7 +2107,7 @@ static void getedgeitems(void)
 	}
 	else {if (S->subg) v = cons_subg(S->subg); S->subg = 0;}
 	/* else nil append */
-	if (v) listapp(&(S->edgelist),v);
+	if (v) listapp(&S->edgelist, v);
 }
 
 static void endedge(void)
@@ -2104,7 +2123,7 @@ static void endedge(void)
 	/* look for "key" pseudo-attribute */
 	key = NULL;
 	for (aptr = S->attrlist.first; aptr; aptr = aptr->next) {
-		if ((aptr->tag == T_atom) && streq(aptr->u.name,Key))
+		if (aptr->tag == T_atom && streq(aptr->u.name,Key))
 			key = aptr->str;
 	}
 
@@ -2120,9 +2139,9 @@ static void endedge(void)
 				edgerhs(tptr->u.n,tptr->str,p->next,key);
 		}
 	}
-	deletelist(&(S->nodelist));
-	deletelist(&(S->edgelist));
-	deletelist(&(S->attrlist));
+	deletelist(&S->nodelist);
+	deletelist(&S->edgelist);
+	deletelist(&S->attrlist);
 	S->subg = 0;
 }
 
@@ -2196,7 +2215,7 @@ static void newedge(Agnode_t *t, char *tport, Agnode_t *h, char *hport, char *ke
 	if (e) {		/* can fail if graph is strict and t==h */
 		char    *tp = tport;
 		char    *hp = hport;
-		if ((agtail(e) != aghead(e)) && (aghead(e) == t)) {
+		if (agtail(e) != aghead(e) && aghead(e) == t) {
 			/* could happen with an undirected edge */
 			char    *temp;
 			temp = tp; tp = hp; hp = temp;
@@ -2251,9 +2270,9 @@ static void closesubg(void)
 static void freestack(void)
 {
 	while (S) {
-		deletelist(&(S->nodelist));
-		deletelist(&(S->attrlist));
-		deletelist(&(S->edgelist));
+		deletelist(&S->nodelist);
+		deletelist(&S->attrlist);
+		deletelist(&S->edgelist);
 		S = pop(S);
 	}
 }
