@@ -1,9 +1,11 @@
+/// @file
+/// @ingroup common_utils
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -14,16 +16,17 @@
 #include "htmlparse.h"
 #include <common/htmllex.h>
 #include <cdt/cdt.h>
-#include <ctype.h>
-#include <cgraph/alloc.h>
-#include <cgraph/strcasecmp.h>
+#include <cgraph/gv_ctype.h>
 #include <cgraph/strview.h>
 #include <cgraph/tokenize.h>
-#include <cgraph/unused.h>
 #include <limits.h>
 #include <stdbool.h>
 #include <stddef.h>
 #include <stdint.h>
+#include <util/alloc.h>
+#include <util/startswith.h>
+#include <util/strcasecmp.h>
+#include <util/unused.h>
 
 #ifdef HAVE_EXPAT
 #ifdef _WIN32
@@ -38,59 +41,39 @@
 #define XML_STATUS_ERROR 0
 #endif
 
-typedef struct {
-#ifdef HAVE_EXPAT
-    XML_Parser parser;
-#endif
-    char* ptr;         // input source
-    int tok;           // token type
-    agxbuf* xb;        // buffer to gather T_string data
-    agxbuf lb;         // buffer for translating lexical data
-    int warn;          // set if warning given
-    int error;         // set if error given
-    char inCell;       // set if in TD to allow T_string
-    char mode;         // for handling artificial <HTML>..</HTML>
-    char *currtok;     // for error reporting
-    char *prevtok;     // for error reporting
-    size_t currtoklen;
-    size_t prevtoklen;
-} lexstate_t;
-static lexstate_t state;
+static unsigned long htmllineno_ctx(htmllexstate_t *ctx);
 
 /* error_context:
  * Print the last 2 "token"s seen.
  */
-static void error_context(void)
+static void error_context(htmllexstate_t *ctx)
 {
-    agxbclear(state.xb);
-    if (state.prevtoklen > 0)
-	agxbput_n(state.xb, state.prevtok, state.prevtoklen);
-    agxbput_n(state.xb, state.currtok, state.currtoklen);
-    agerr(AGPREV, "... %s ...\n", agxbuse(state.xb));
+  agerr(AGPREV, "... %.*s%.*s ...\n", (int)ctx->prevtok.size,
+        ctx->prevtok.data, (int)ctx->currtok.size, ctx->currtok.data);
 }
 
 /* htmlerror:
  * yyerror - called by yacc output
  */
-void htmlerror(const char *msg)
+void htmlerror(htmlscan_t *scanner, const char *msg)
 {
-    if (state.error)
+    htmllexstate_t *ctx = &scanner->lexer;
+    if (ctx->error)
 	return;
-    state.error = 1;
-    agerr(AGERR, "%s in line %d \n", msg, htmllineno());
-    error_context();
+    ctx->error = 1;
+    agerrorf("%s in line %lu \n", msg, htmllineno(scanner));
+    error_context(&scanner->lexer);
 }
 
 #ifdef HAVE_EXPAT
 /* lexerror:
  * called by lexer when unknown <..> is found.
  */
-static void lexerror(const char *name)
+static void lexerror(htmllexstate_t *ctx, const char *name)
 {
-    state.tok = T_error;
-    state.error = 1;
-    agerr(AGERR, "Unknown HTML element <%s> on line %d \n",
-	  name, htmllineno());
+    ctx->tok = T_error;
+    ctx->error = 1;
+    agerrorf("Unknown HTML element <%s> on line %lu \n", name, htmllineno_ctx(ctx));
 }
 
 typedef int (*attrFn) (void *, char *);
@@ -105,11 +88,11 @@ typedef struct {
 #define ISIZE (sizeof(attr_item))
 
 /* icmp:
- * Compare two attr_item. Used in bsearch
+ * Compare an attr_item. Used in bsearch
  */
-static int icmp(attr_item * i, attr_item * j)
-{
-    return strcasecmp(i->name, j->name);
+static int icmp(const void *name, const void *item) {
+  const attr_item *j = item;
+  return strcasecmp(name, j->name);
 }
 
 static int bgcolorfn(htmldata_t * p, char *v)
@@ -136,7 +119,7 @@ static int sidesfn(htmldata_t * p, char *v)
     char c;
 
     while ((c = *v++)) {
-	switch (tolower(c)) {
+	switch (gv_tolower(c)) {
 	case 'l' :
 	    flags |= BORDER_LEFT;
 	    break;
@@ -150,7 +133,7 @@ static int sidesfn(htmldata_t * p, char *v)
 	    flags |= BORDER_BOTTOM;
 	    break;
 	default :
-	    agerr(AGWARN, "Unrecognized character '%c' (%d) in sides attribute\n", c, c);
+	    agwarningf("Unrecognized character '%c' (%d) in sides attribute\n", c, c);
 	    break;
 	}
     }
@@ -178,15 +161,17 @@ static int stylefn(htmldata_t * p, char *v)
     int rv = 0;
     for (tok_t t = tok(v, DELIM); !tok_end(&t); tok_next(&t)) {
 	strview_t tk = tok_get(&t);
-	if (strview_case_str_eq(tk, "ROUNDED")) p->style |= ROUNDED;
-	else if (strview_case_str_eq(tk, "RADIAL")) p->style |= RADIAL;
-	else if (strview_case_str_eq(tk,"SOLID")) p->style &= (unsigned short)~(DOTTED|DASHED);
-	else if (strview_case_str_eq(tk,"INVISIBLE") ||
-	         strview_case_str_eq(tk,"INVIS")) p->style |= INVISIBLE;
-	else if (strview_case_str_eq(tk,"DOTTED")) p->style |= DOTTED;
-	else if (strview_case_str_eq(tk,"DASHED")) p->style |= DASHED;
+	if (strview_case_str_eq(tk, "ROUNDED")) p->style.rounded = true;
+	else if (strview_case_str_eq(tk, "RADIAL")) p->style.radial = true;
+	else if (strview_case_str_eq(tk,"SOLID")) {
+	    p->style.dotted = false;
+	    p->style.dashed = false;
+	} else if (strview_case_str_eq(tk,"INVISIBLE") ||
+	           strview_case_str_eq(tk,"INVIS")) p->style.invisible = true;
+	else if (strview_case_str_eq(tk,"DOTTED")) p->style.dotted = true;
+	else if (strview_case_str_eq(tk,"DASHED")) p->style.dashed = true;
 	else {
-	    agerr(AGWARN, "Illegal value %.*s for STYLE - ignored\n", (int)tk.size,
+	    agwarningf("Illegal value %.*s for STYLE - ignored\n", (int)tk.size,
 	          tk.data);
 	    rv = 1;
 	}
@@ -220,13 +205,13 @@ static int doInt(char *v, char *s, int min, int max, long *ul)
     long b = strtol(v, &ep, 10);
 
     if (ep == v) {
-	agerr(AGWARN, "Improper %s value %s - ignored", s, v);
+	agwarningf("Improper %s value %s - ignored", s, v);
 	rv = 1;
     } else if (b > max) {
-	agerr(AGWARN, "%s value %s > %d - too large - ignored", s, v, max);
+	agwarningf("%s value %s > %d - too large - ignored", s, v, max);
 	rv = 1;
     } else if (b < min) {
-	agerr(AGWARN, "%s value %s < %d - too small - ignored", s, v, min);
+	agwarningf("%s value %s < %d - too small - ignored", s, v, min);
 	rv = 1;
     } else
 	*ul = b;
@@ -291,7 +276,7 @@ static int cellborderfn(htmltbl_t * p, char *v)
 static int columnsfn(htmltbl_t * p, char *v)
 {
     if (*v != '*') {
-	agerr(AGWARN, "Unknown value %s for COLUMNS - ignored\n", v);
+	agwarningf("Unknown value %s for COLUMNS - ignored\n", v);
 	return 1;
     }
     p->vrule = true;
@@ -301,7 +286,7 @@ static int columnsfn(htmltbl_t * p, char *v)
 static int rowsfn(htmltbl_t * p, char *v)
 {
     if (*v != '*') {
-	agerr(AGWARN, "Unknown value %s for ROWS - ignored\n", v);
+	agwarningf("Unknown value %s for ROWS - ignored\n", v);
 	return 1;
     }
     p->hrule = true;
@@ -314,7 +299,7 @@ static int fixedsizefn(htmldata_t * p, char *v)
     if (!strcasecmp(v, "TRUE"))
 	p->flags |= FIXED_FLAG;
     else if (strcasecmp(v, "FALSE")) {
-	agerr(AGWARN, "Illegal value %s for FIXEDSIZE - ignored\n", v);
+	agwarningf("Illegal value %s for FIXEDSIZE - ignored\n", v);
 	rv = 1;
     }
     return rv;
@@ -328,7 +313,7 @@ static int valignfn(htmldata_t * p, char *v)
     else if (!strcasecmp(v, "TOP"))
 	p->flags |= VALIGN_TOP;
     else if (strcasecmp(v, "MIDDLE")) {
-	agerr(AGWARN, "Illegal value %s for VALIGN - ignored\n", v);
+	agwarningf("Illegal value %s for VALIGN - ignored\n", v);
 	rv = 1;
     }
     return rv;
@@ -342,7 +327,7 @@ static int halignfn(htmldata_t * p, char *v)
     else if (!strcasecmp(v, "RIGHT"))
 	p->flags |= HALIGN_RIGHT;
     else if (strcasecmp(v, "CENTER")) {
-	agerr(AGWARN, "Illegal value %s for ALIGN - ignored\n", v);
+	agwarningf("Illegal value %s for ALIGN - ignored\n", v);
 	rv = 1;
     }
     return rv;
@@ -360,7 +345,7 @@ static int cell_halignfn(htmldata_t * p, char *v)
     else if (strcasecmp(v, "CENTER"))
 	rv = 1;
     if (rv)
-	agerr(AGWARN, "Illegal value %s for ALIGN in TD - ignored\n", v);
+	agwarningf("Illegal value %s for ALIGN in TD - ignored\n", v);
     return rv;
 }
 
@@ -374,7 +359,7 @@ static int balignfn(htmldata_t * p, char *v)
     else if (strcasecmp(v, "CENTER"))
 	rv = 1;
     if (rv)
-	agerr(AGWARN, "Illegal value %s for BALIGN in TD - ignored\n", v);
+	agwarningf("Illegal value %s for BALIGN in TD - ignored\n", v);
     return rv;
 }
 
@@ -405,7 +390,7 @@ static int rowspanfn(htmlcell_t * p, char *v)
     if (doInt(v, "ROWSPAN", 0, UINT16_MAX, &u))
 	return 1;
     if (u == 0) {
-	agerr(AGWARN, "ROWSPAN value cannot be 0 - ignored\n");
+	agwarningf("ROWSPAN value cannot be 0 - ignored\n");
 	return 1;
     }
     p->rowspan = (uint16_t)u;
@@ -419,7 +404,7 @@ static int colspanfn(htmlcell_t * p, char *v)
     if (doInt(v, "COLSPAN", 0, UINT16_MAX, &u))
 	return 1;
     if (u == 0) {
-	agerr(AGWARN, "COLSPAN value cannot be 0 - ignored\n");
+	agwarningf("COLSPAN value cannot be 0 - ignored\n");
 	return 1;
     }
     p->colspan = (uint16_t)u;
@@ -470,7 +455,7 @@ static int alignfn(int *p, char *v)
     else if (!strcasecmp(v, "CENTER"))
 	*p = 'n';
     else {
-	agerr(AGWARN, "Illegal value %s for ALIGN - ignored\n", v);
+	agwarningf("Illegal value %s for ALIGN - ignored\n", v);
 	rv = 1;
     }
     return rv;
@@ -550,44 +535,41 @@ static attr_item br_items[] = {
  * Name/value pairs are in array atts, which is null terminated.
  * s is the name of the HTML element being processed.
  */
-static void
-doAttrs(void *tp, attr_item * items, int nel, char **atts, char *s)
-{
+static void doAttrs(htmllexstate_t *ctx, void *tp, attr_item *items, size_t nel, char **atts,
+                    char *s) {
     char *name;
     char *val;
     attr_item *ip;
-    attr_item key;
 
     while ((name = *atts++) != NULL) {
 	val = *atts++;
-	key.name = name;
-	ip = bsearch(&key, items, nel, ISIZE, (bcmpfn) icmp);
+	ip = bsearch(name, items, nel, ISIZE, icmp);
 	if (ip)
-	    state.warn |= ip->action(tp, val);
+	    ctx->warn |= ip->action(tp, val);
 	else {
-	    agerr(AGWARN, "Illegal attribute %s in %s - ignored\n", name,
+	    agwarningf("Illegal attribute %s in %s - ignored\n", name,
 		  s);
-	    state.warn = 1;
+	    ctx->warn = 1;
 	}
     }
 }
 
-static void mkBR(char **atts)
+static void mkBR(htmllexstate_t *ctx, char **atts)
 {
-    htmllval.i = UNSET_ALIGN;
-    doAttrs(&htmllval.i, br_items, sizeof(br_items) / ISIZE, atts, "<BR>");
+    ctx->htmllval->i = UNSET_ALIGN;
+    doAttrs(ctx, &ctx->htmllval->i, br_items, sizeof(br_items) / ISIZE, atts, "<BR>");
 }
 
-static htmlimg_t *mkImg(char **atts)
+static htmlimg_t *mkImg(htmllexstate_t *ctx, char **atts)
 {
     htmlimg_t *img = gv_alloc(sizeof(htmlimg_t));
 
-    doAttrs(img, img_items, sizeof(img_items) / ISIZE, atts, "<IMG>");
+    doAttrs(ctx, img, img_items, sizeof(img_items) / ISIZE, atts, "<IMG>");
 
     return img;
 }
 
-static textfont_t *mkFont(GVC_t *gvc, char **atts, unsigned char flags) {
+static textfont_t *mkFont(htmllexstate_t *ctx, char **atts, unsigned char flags) {
     textfont_t tf = {NULL,NULL,NULL,0.0,0,0};
 
     tf.size = -1.0;		/* unassigned */
@@ -595,141 +577,141 @@ static textfont_t *mkFont(GVC_t *gvc, char **atts, unsigned char flags) {
     assert(flags <= FLAGS_MAX);
     tf.flags = (unsigned char)(flags & FLAGS_MAX);
     if (atts)
-	doAttrs(&tf, font_items, sizeof(font_items) / ISIZE, atts, "<FONT>");
+	doAttrs(ctx, &tf, font_items, sizeof(font_items) / ISIZE, atts, "<FONT>");
 
-    return dtinsert(gvc->textfont_dt, &tf);
+    return dtinsert(ctx->gvc->textfont_dt, &tf);
 }
 
-static htmlcell_t *mkCell(char **atts)
+static htmlcell_t *mkCell(htmllexstate_t *ctx, char **atts)
 {
     htmlcell_t *cell = gv_alloc(sizeof(htmlcell_t));
 
     cell->colspan = 1;
     cell->rowspan = 1;
-    doAttrs(cell, cell_items, sizeof(cell_items) / ISIZE, atts, "<TD>");
+    doAttrs(ctx, cell, cell_items, sizeof(cell_items) / ISIZE, atts, "<TD>");
 
     return cell;
 }
 
-static htmltbl_t *mkTbl(char **atts)
+static htmltbl_t *mkTbl(htmllexstate_t *ctx, char **atts)
 {
     htmltbl_t *tbl = gv_alloc(sizeof(htmltbl_t));
 
     tbl->row_count = SIZE_MAX; // flag that table is a raw, parsed table
     tbl->cellborder = -1; // unset cell border attribute
-    doAttrs(tbl, tbl_items, sizeof(tbl_items) / ISIZE, atts, "<TABLE>");
+    doAttrs(ctx, tbl, tbl_items, sizeof(tbl_items) / ISIZE, atts, "<TABLE>");
 
     return tbl;
 }
 
 static void startElement(void *user, const char *name, char **atts)
 {
-    GVC_t *gvc = user;
+    htmllexstate_t *ctx = user;
 
     if (strcasecmp(name, "TABLE") == 0) {
-	htmllval.tbl = mkTbl(atts);
-	state.inCell = 0;
-	state.tok = T_table;
+	ctx->htmllval->tbl = mkTbl(ctx, atts);
+	ctx->inCell = 0;
+	ctx->tok = T_table;
     } else if (strcasecmp(name, "TR") == 0 || strcasecmp(name, "TH") == 0) {
-	state.inCell = 0;
-	state.tok = T_row;
+	ctx->inCell = 0;
+	ctx->tok = T_row;
     } else if (strcasecmp(name, "TD") == 0) {
-	state.inCell = 1;
-	htmllval.cell = mkCell(atts);
-	state.tok = T_cell;
+	ctx->inCell = 1;
+	ctx->htmllval->cell = mkCell(ctx, atts);
+	ctx->tok = T_cell;
     } else if (strcasecmp(name, "FONT") == 0) {
-	htmllval.font = mkFont(gvc, atts, 0);
-	state.tok = T_font;
+	ctx->htmllval->font = mkFont(ctx, atts, 0);
+	ctx->tok = T_font;
     } else if (strcasecmp(name, "B") == 0) {
-	htmllval.font = mkFont(gvc, 0, HTML_BF);
-	state.tok = T_bold;
+	ctx->htmllval->font = mkFont(ctx, 0, HTML_BF);
+	ctx->tok = T_bold;
     } else if (strcasecmp(name, "S") == 0) {
-	htmllval.font = mkFont(gvc, 0, HTML_S);
-	state.tok = T_s;
+	ctx->htmllval->font = mkFont(ctx, 0, HTML_S);
+	ctx->tok = T_s;
     } else if (strcasecmp(name, "U") == 0) {
-	htmllval.font = mkFont(gvc, 0, HTML_UL);
-	state.tok = T_underline;
+	ctx->htmllval->font = mkFont(ctx, 0, HTML_UL);
+	ctx->tok = T_underline;
     } else if (strcasecmp(name, "O") == 0) {
-	htmllval.font = mkFont(gvc, 0, HTML_OL);
-	state.tok = T_overline;
+	ctx->htmllval->font = mkFont(ctx, 0, HTML_OL);
+	ctx->tok = T_overline;
     } else if (strcasecmp(name, "I") == 0) {
-	htmllval.font = mkFont(gvc, 0, HTML_IF);
-	state.tok = T_italic;
+	ctx->htmllval->font = mkFont(ctx, 0, HTML_IF);
+	ctx->tok = T_italic;
     } else if (strcasecmp(name, "SUP") == 0) {
-	htmllval.font = mkFont(gvc, 0, HTML_SUP);
-	state.tok = T_sup;
+	ctx->htmllval->font = mkFont(ctx, 0, HTML_SUP);
+	ctx->tok = T_sup;
     } else if (strcasecmp(name, "SUB") == 0) {
-	htmllval.font = mkFont(gvc, 0, HTML_SUB);
-	state.tok = T_sub;
+	ctx->htmllval->font = mkFont(ctx, 0, HTML_SUB);
+	ctx->tok = T_sub;
     } else if (strcasecmp(name, "BR") == 0) {
-	mkBR(atts);
-	state.tok = T_br;
+	mkBR(ctx, atts);
+	ctx->tok = T_br;
     } else if (strcasecmp(name, "HR") == 0) {
-	state.tok = T_hr;
+	ctx->tok = T_hr;
     } else if (strcasecmp(name, "VR") == 0) {
-	state.tok = T_vr;
+	ctx->tok = T_vr;
     } else if (strcasecmp(name, "IMG") == 0) {
-	htmllval.img = mkImg(atts);
-	state.tok = T_img;
+	ctx->htmllval->img = mkImg(ctx, atts);
+	ctx->tok = T_img;
     } else if (strcasecmp(name, "HTML") == 0) {
-	state.tok = T_html;
+	ctx->tok = T_html;
     } else {
-	lexerror(name);
+	lexerror(ctx, name);
     }
 }
 
 static void endElement(void *user, const char *name)
 {
-    (void)user;
+    htmllexstate_t *ctx = user;
 
     if (strcasecmp(name, "TABLE") == 0) {
-	state.tok = T_end_table;
-	state.inCell = 1;
+	ctx->tok = T_end_table;
+	ctx->inCell = 1;
     } else if (strcasecmp(name, "TR") == 0 || strcasecmp(name, "TH") == 0) {
-	state.tok = T_end_row;
+	ctx->tok = T_end_row;
     } else if (strcasecmp(name, "TD") == 0) {
-	state.tok = T_end_cell;
-	state.inCell = 0;
+	ctx->tok = T_end_cell;
+	ctx->inCell = 0;
     } else if (strcasecmp(name, "HTML") == 0) {
-	state.tok = T_end_html;
+	ctx->tok = T_end_html;
     } else if (strcasecmp(name, "FONT") == 0) {
-	state.tok = T_end_font;
+	ctx->tok = T_end_font;
     } else if (strcasecmp(name, "B") == 0) {
-	state.tok = T_n_bold;
+	ctx->tok = T_n_bold;
     } else if (strcasecmp(name, "U") == 0) {
-	state.tok = T_n_underline;
+	ctx->tok = T_n_underline;
     } else if (strcasecmp(name, "O") == 0) {
-	state.tok = T_n_overline;
+	ctx->tok = T_n_overline;
     } else if (strcasecmp(name, "I") == 0) {
-	state.tok = T_n_italic;
+	ctx->tok = T_n_italic;
     } else if (strcasecmp(name, "SUP") == 0) {
-	state.tok = T_n_sup;
+	ctx->tok = T_n_sup;
     } else if (strcasecmp(name, "SUB") == 0) {
-	state.tok = T_n_sub;
+	ctx->tok = T_n_sub;
     } else if (strcasecmp(name, "S") == 0) {
-	state.tok = T_n_s;
+	ctx->tok = T_n_s;
     } else if (strcasecmp(name, "BR") == 0) {
-	if (state.tok == T_br)
-	    state.tok = T_BR;
+	if (ctx->tok == T_br)
+	    ctx->tok = T_BR;
 	else
-	    state.tok = T_end_br;
+	    ctx->tok = T_end_br;
     } else if (strcasecmp(name, "HR") == 0) {
-	if (state.tok == T_hr)
-	    state.tok = T_HR;
+	if (ctx->tok == T_hr)
+	    ctx->tok = T_HR;
 	else
-	    state.tok = T_end_hr;
+	    ctx->tok = T_end_hr;
     } else if (strcasecmp(name, "VR") == 0) {
-	if (state.tok == T_vr)
-	    state.tok = T_VR;
+	if (ctx->tok == T_vr)
+	    ctx->tok = T_VR;
 	else
-	    state.tok = T_end_vr;
+	    ctx->tok = T_end_vr;
     } else if (strcasecmp(name, "IMG") == 0) {
-	if (state.tok == T_img)
-	    state.tok = T_IMG;
+	if (ctx->tok == T_img)
+	    ctx->tok = T_IMG;
 	else
-	    state.tok = T_end_img;
+	    ctx->tok = T_end_img;
     } else {
-	lexerror(name);
+	lexerror(ctx, name);
     }
 }
 
@@ -742,47 +724,50 @@ static void endElement(void *user, const char *name)
  */
 static void characterData(void *user, const char *s, int length)
 {
-    (void)user;
+    htmllexstate_t *ctx = user;
 
     int i, cnt = 0;
     unsigned char c;
 
-    if (state.inCell) {
+    if (ctx->inCell) {
 	for (i = length; i; i--) {
 	    c = *s++;
 	    if (c >= ' ') {
 		cnt++;
-		agxbputc(state.xb, (char)c);
+		agxbputc(ctx->xb, (char)c);
 	    }
 	}
-	if (cnt) state.tok = T_string;
+	if (cnt) ctx->tok = T_string;
     }
 }
 #endif
 
-int initHTMLlexer(char *src, agxbuf * xb, htmlenv_t *env)
+int initHTMLlexer(htmlscan_t *scanner, char *src, agxbuf * xb, htmlenv_t *env)
 {
 #ifdef HAVE_EXPAT
-    state.xb = xb;
-    state.lb = (agxbuf){0};
-    state.ptr = src;
-    state.mode = 0;
-    state.warn = 0;
-    state.error = 0;
-    state.currtoklen = 0;
-    state.prevtoklen = 0;
-    state.inCell = 1;
-    state.parser = XML_ParserCreate(charsetToStr(GD_charset(env->g)));
-    XML_SetUserData(state.parser, GD_gvc(env->g));
-    XML_SetElementHandler(state.parser,
+    htmllexstate_t *ctx = &scanner->lexer;
+
+    ctx->xb = xb;
+    ctx->lb = (agxbuf){0};
+    ctx->ptr = src;
+    ctx->mode = 0;
+    ctx->warn = 0;
+    ctx->error = 0;
+    ctx->currtok = (strview_t){0};
+    ctx->prevtok = (strview_t){0};
+    ctx->inCell = 1;
+    ctx->parser = XML_ParserCreate(charsetToStr(GD_charset(env->g)));
+    ctx->gvc = GD_gvc(env->g);
+    XML_SetUserData(ctx->parser, ctx);
+    XML_SetElementHandler(ctx->parser,
 			  (XML_StartElementHandler) startElement,
 			  endElement);
-    XML_SetCharacterDataHandler(state.parser, characterData);
+    XML_SetCharacterDataHandler(ctx->parser, characterData);
     return 0;
 #else
     static int first;
     if (!first) {
-	agerr(AGWARN,
+	agwarningf(
 	      "Not built with libexpat. Table formatting is not available.\n");
 	first++;
     }
@@ -790,12 +775,13 @@ int initHTMLlexer(char *src, agxbuf * xb, htmlenv_t *env)
 #endif
 }
 
-int clearHTMLlexer(void)
+int clearHTMLlexer(htmlscan_t *scanner)
 {
 #ifdef HAVE_EXPAT
-    int rv = state.error ? 3 : state.warn;
-    XML_ParserFree(state.parser);
-    agxbfree (&state.lb);
+    htmllexstate_t *ctx = &scanner->lexer;
+    int rv = ctx->error ? 3 : ctx->warn;
+    XML_ParserFree(ctx->parser);
+    agxbfree (&ctx->lb);
     return rv;
 #else
     return 1;
@@ -818,7 +804,7 @@ static UNUSED void agxbput_move(agxbuf *dst, const char *src) {
  * or null character otherwise.
  * We rely on HTML strings having matched nested <>.
  */
-static char *eatComment(char *p)
+static char *eatComment(htmllexstate_t *ctx, char *p)
 {
     int depth = 1;
     char *s = p;
@@ -833,9 +819,9 @@ static char *eatComment(char *p)
     s--;			/* move back to '\0' or '>' */
     if (*s) {
 	char *t = s - 2;
-	if (t < p || strncmp(t, "--", 2)) {
-	    agerr(AGWARN, "Unclosed comment\n");
-	    state.warn = 1;
+	if (t < p || !startswith(t, "--")) {
+	    agwarningf("Unclosed comment\n");
+	    ctx->warn = 1;
 	}
     }
     return s;
@@ -845,20 +831,20 @@ static char *eatComment(char *p)
  * Return next XML unit. This is either <..>, an HTML 
  * comment <!-- ... -->, or characters up to next <.
  */
-static char *findNext(char *s, agxbuf* xb)
+static char *findNext(htmllexstate_t *ctx, char *s, agxbuf* xb)
 {
     char* t = s + 1;
     char c;
 
     if (*s == '<') {
-	if (!strncmp(t, "!--", 3))
-	    t = eatComment(t + 3);
+	if (startswith(t, "!--"))
+	    t = eatComment(ctx, t + 3);
 	else
 	    while (*t && *t != '>')
 		t++;
 	if (*t != '>') {
-	    agerr(AGWARN, "Label closed before end of HTML element\n");
-	    state.warn = 1;
+	    agwarningf("Label closed before end of HTML element\n");
+	    ctx->warn = 1;
 	} else
 	    t++;
     } else {
@@ -917,17 +903,21 @@ static void protect_rsqb(agxbuf *xb) {
 }
 #endif
 
-int htmllineno(void)
-{
+
+unsigned long htmllineno(htmlscan_t *scanner) {
+    return htmllineno_ctx(&scanner->lexer);
+}
+
+static unsigned long htmllineno_ctx(htmllexstate_t *ctx) {
 #ifdef HAVE_EXPAT
-    return XML_GetCurrentLineNumber(state.parser);
+    return XML_GetCurrentLineNumber(ctx->parser);
 #else
     return 0;
 #endif
 }
 
 #ifdef DEBUG
-static void printTok(int tok)
+static void printTok(htmllexstate_t *ctx, int tok)
 {
     char *s;
 
@@ -1050,16 +1040,16 @@ static void printTok(int tok)
 	s = "<unknown>";
     }
     if (tok == T_string) {
-	const char *token_text = agxbuse(state.xb);
+	const char *token_text = agxbuse(ctx->xb);
 	fprintf(stderr, "%s \"%s\"\n", s, token_text);
-	agxbput_move(state.xb, token_text);
+	agxbput_move(ctx->xb, token_text);
     } else
 	fprintf(stderr, "%s\n", s);
 }
 
 #endif
 
-int htmllex(void)
+int htmllex(union HTMLSTYPE *htmllval, htmlscan_t *scanner)
 {
 #ifdef HAVE_EXPAT
     static char *begin_html = "<HTML>";
@@ -1069,58 +1059,57 @@ int htmllex(void)
     char *endp = 0;
     size_t len, llen;
     int rv;
+    htmllexstate_t *ctx = &scanner->lexer;
 
-    state.tok = 0;
+    ctx->htmllval = htmllval;
+    ctx->tok = 0;
     do {
-	if (state.mode == 2)
+	if (ctx->mode == 2)
 	    return EOF;
-	if (state.mode == 0) {
-	    state.mode = 1;
+	if (ctx->mode == 0) {
+	    ctx->mode = 1;
 	    s = begin_html;
 	    len = strlen(s);
 	    endp = 0;
 	} else {
-	    s = state.ptr;
+	    s = ctx->ptr;
 	    if (*s == '\0') {
-		state.mode = 2;
+		ctx->mode = 2;
 		s = end_html;
 		len = strlen(s);
 	    } else {
-		endp = findNext(s,&state.lb);
+		endp = findNext(ctx, s,&ctx->lb);
 		len = (size_t)(endp - s);
 	    }
 	}
 
-	protect_rsqb(&state.lb);
+	protect_rsqb(&ctx->lb);
 
-	state.prevtok = state.currtok;
-	state.prevtoklen = state.currtoklen;
-	state.currtok = s;
-	state.currtoklen = len;
-	if ((llen = (size_t)agxblen(&state.lb))) {
+	ctx->prevtok = ctx->currtok;
+	ctx->currtok = (strview_t){.data = s, .size = len};
+	if ((llen = agxblen(&ctx->lb))) {
 	    assert(llen <= (size_t)INT_MAX && "XML token too long for expat API");
-	    rv = XML_Parse(state.parser, agxbuse(&state.lb), (int)llen, 0);
+	    rv = XML_Parse(ctx->parser, agxbuse(&ctx->lb), (int)llen, 0);
 	} else {
 	    assert(len <= (size_t)INT_MAX && "XML token too long for expat API");
-	    rv = XML_Parse(state.parser, s, (int)len, len ? 0 : 1);
+	    rv = XML_Parse(ctx->parser, s, (int)len, len ? 0 : 1);
 	}
 	if (rv == XML_STATUS_ERROR) {
-	    if (!state.error) {
-		agerr(AGERR, "%s in line %d \n",
-		      XML_ErrorString(XML_GetErrorCode(state.parser)),
-		      htmllineno());
-		error_context();
-		state.error = 1;
-		state.tok = T_error;
+	    if (!ctx->error) {
+		agerrorf("%s in line %lu \n",
+		      XML_ErrorString(XML_GetErrorCode(ctx->parser)), htmllineno(scanner));
+		error_context(ctx);
+		ctx->error = 1;
+		ctx->tok = T_error;
 	    }
 	}
 	if (endp)
-	    state.ptr = endp;
-    } while (state.tok == 0);
+	    ctx->ptr = endp;
+    } while (ctx->tok == 0);
 #ifdef DEBUG
-    printTok (state.tok);
+    printTok (ctx, ctx->tok);
 #endif
-    return state.tok;
+    return ctx->tok;
 #else
     return EOF;
 #endif
