@@ -1,51 +1,55 @@
+/// @file
+/// @ingroup common_render
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-
-/* Implementation of HTML-like tables.
- * 
- * The (now purged) CodeGen graphics model, especially with integral coodinates, is
- * not adequate to handle this as we would like. In particular, it is
- * difficult to handle notions of adjacency and correct rounding to pixels.
- * For example, if 2 adjacent boxes bb1.UR.x == bb2.LL.x, the rectangles
- * may be drawn overlapping. However, if we use bb1.UR.x+1 == bb2.LL.x
- * there may or may not be a gap between them, even in the same device
- * depending on their positions. When CELLSPACING > 1, this isn't as much
- * of a problem.
- *
- * We allow negative spacing as a hack to allow overlapping cell boundaries.
- * For the reasons discussed above, this is difficult to get correct.
- * This is an important enough case we should extend the table model to
- * support it correctly. This could be done by allowing a table attribute,
- * e.g., CELLGRID=n, which sets CELLBORDER=0 and has the border drawing
- * handled correctly by the table.
- */
+// Implementation of HTML-like tables.
+//
+// The (now purged) CodeGen graphics model, especially with integral
+// coordinates, is not adequate to handle this as we would like. In particular,
+// it is difficult to handle notions of adjacency and correct rounding to
+// pixels. For example, if 2 adjacent boxes bb1.UR.x == bb2.LL.x, the rectangles
+// may be drawn overlapping. However, if we use bb1.UR.x+1 == bb2.LL.x
+// there may or may not be a gap between them, even in the same device
+// depending on their positions. When CELLSPACING > 1, this isn't as much
+// of a problem.
+//
+// We allow negative spacing as a hack to allow overlapping cell boundaries.
+// For the reasons discussed above, this is difficult to get correct.
+// This is an important enough case we should extend the table model to
+// support it correctly. This could be done by allowing a table attribute,
+// e.g., CELLGRID=n, which sets CELLBORDER=0 and has the border drawing
+// handled correctly by the table.
 
 #include <assert.h>
 #include <common/render.h>
 #include <common/htmltable.h>
-#include <cgraph/agxbuf.h>
-#include <cgraph/prisize_t.h>
+#include <cgraph/gv_math.h>
 #include <common/pointset.h>
-#include <common/intset.h>
 #include <cdt/cdt.h>
-#include <cgraph/alloc.h>
-#include <cgraph/exit.h>
-#include <cgraph/strcasecmp.h>
-#include <cgraph/unreachable.h>
 #include <float.h>
 #include <inttypes.h>
+#include <limits.h>
 #include <math.h>
 #include <stddef.h>
 #include <stdbool.h>
+#include <stdint.h>
 #include <stdio.h>
+#include <util/agxbuf.h>
+#include <util/alloc.h>
+#include <util/bitarray.h>
+#include <util/exit.h>
+#include <util/prisize_t.h>
+#include <util/strcasecmp.h>
+#include <util/streq.h>
+#include <util/unreachable.h>
 
 #define DEFAULT_BORDER    1
 #define DEFAULT_CELLPADDING  2
@@ -65,8 +69,7 @@ typedef struct {
 static void printCell(htmlcell_t * cp, int ind);
 #endif
 
-/* pushFontInfo:
- * Replace current font attributes in env with ones from fp,
+/* Replace current font attributes in env with ones from fp,
  * storing old attributes in savp. We only deal with attributes
  * set in env. The attributes are restored via popFontInfo.
  */
@@ -96,8 +99,7 @@ pushFontInfo(htmlenv_t * env, textfont_t * fp, textfont_t * savp)
     }
 }
 
-/* popFontInfo:
- * Restore saved font attributes.
+/* Restore saved font attributes.
  * Copy only set values.
  */
 static void popFontInfo(htmlenv_t * env, textfont_t * savp)
@@ -217,8 +219,7 @@ static void doSide(GVJ_t * job, pointf p, double wd, double ht)
     gvrender_box(job, BF, 1);
 }
 
-/* mkPts:
- * Convert boxf into four corner points
+/* Convert boxf into four corner points
  * If border is > 1, inset the points by half the border.
  * It is assumed AF is pointf[4], so the data is store there
  * and AF is returned.
@@ -228,7 +229,7 @@ static pointf *mkPts(pointf * AF, boxf b, int border)
     AF[0] = b.LL;
     AF[2] = b.UR;
     if (border > 1) {
-	double delta = (double)border / 2.0;
+	const double delta = border / 2.0;
 	AF[0].x += delta;
 	AF[0].y += delta;
 	AF[2].x -= delta;
@@ -242,8 +243,7 @@ static pointf *mkPts(pointf * AF, boxf b, int border)
     return AF;
 }
 
-/* doBorder:
- * Draw a rectangular border for the box b.
+/* Draw a rectangular border for the box b.
  * Handles dashed and dotted styles, rounded corners.
  * Also handles thick lines.
  * Assume dp->border > 0
@@ -256,19 +256,20 @@ static void doBorder(GVJ_t * job, htmldata_t * dp, boxf b)
     unsigned short sides;
 
     gvrender_set_pencolor(job, color);
-    if (dp->style & (DASHED | DOTTED)) {
+    if (dp->style.dashed || dp->style.dotted) {
 	sptr[0] = sptr[1] = NULL;
-	if (dp->style & DASHED)
+	if (dp->style.dashed)
 	    sptr[0] = "dashed";
-	else if (dp->style & DOTTED)
+	else if (dp->style.dotted)
 	    sptr[0] = "dotted";
 	gvrender_set_style(job, sptr);
     } else
 	gvrender_set_style(job, job->gvc->defaultlinestyle);
     gvrender_set_penwidth(job, dp->border);
 
-    if (dp->style & ROUNDED)
-	round_corners(job, mkPts(AF, b, dp->border), 4, ROUNDED, 0);
+    if (dp->style.rounded)
+	round_corners(job, mkPts(AF, b, dp->border), 4,
+	              (graphviz_polygon_style_t){.rounded = true}, 0);
     else if ((sides = (dp->flags & BORDER_MASK))) {
 	mkPts (AF+1, b, dp->border);  /* AF[1-4] has LL=SW,SE,UR=NE,NW */
 	switch (sides) {
@@ -324,10 +325,12 @@ static void doBorder(GVJ_t * job, htmldata_t * dp, boxf b)
 	    gvrender_polyline(job, AF, 2);
 	    gvrender_polyline(job, AF+2, 2);
 	    break;
+	default:
+	    break;
 	}
     } else {
 	if (dp->border > 1) {
-	    double delta = (double)dp->border / 2.0;
+	    const double delta = dp->border / 2.0;
 	    b.LL.x += delta;
 	    b.LL.y += delta;
 	    b.UR.x -= delta;
@@ -337,22 +340,20 @@ static void doBorder(GVJ_t * job, htmldata_t * dp, boxf b)
     }
 }
 
-/* setFill:
- * Set up fill values from given color; make pen transparent.
+/* Set up fill values from given color; make pen transparent.
  * Return type of fill required.
  */
-static int
-setFill(GVJ_t * job, char *color, int angle, int style, char *clrs[2])
-{
+static int setFill(GVJ_t *job, char *color, int angle, htmlstyle_t style,
+                   char *clrs[2]) {
     int filled;
-    float frac;
+    double frac;
     if (findStopColor(color, clrs, &frac)) {
 	gvrender_set_fillcolor(job, clrs[0]);
 	if (clrs[1])
 	    gvrender_set_gradient_vals(job, clrs[1], angle, frac);
 	else
 	    gvrender_set_gradient_vals(job, DEFAULT_COLOR, angle, frac);
-	if (style & RADIAL)
+	if (style.radial)
 	    filled = RGRADIENT;
 	else
 	    filled = GRADIENT;
@@ -364,8 +365,7 @@ setFill(GVJ_t * job, char *color, int angle, int style, char *clrs[2])
     return filled;
 }
 
-/* initAnchor:
- * Save current map values.
+/* Save current map values.
  * Initialize fields in job->obj pertaining to anchors.
  * In particular, this also sets the output rectangle.
  * If there is something to do, 
@@ -418,8 +418,7 @@ initAnchor(GVJ_t * job, htmlenv_t * env, htmldata_t * data, boxf b,
 #define RESET(fld) \
   if(obj->fld != save->fld) {free(obj->fld); obj->fld = save->fld;}
 
-/* endAnchor:
- * Pop context pushed by initAnchor.
+/* Pop context pushed by initAnchor.
  * This is done by ending current anchor, restoring old values and
  * freeing new.
  *
@@ -442,11 +441,9 @@ static void endAnchor(GVJ_t * job, htmlmap_data_t * save)
     obj->explicit_tooltip = save->explicit_tooltip;
 }
 
-/* forward declaration */
 static void emit_html_cell(GVJ_t * job, htmlcell_t * cp, htmlenv_t * env);
 
-/* emit_html_rules:
- * place vertical and horizontal lines between adjacent cells and
+/* place vertical and horizontal lines between adjacent cells and
  * extend the lines to intersect the rounded table boundary 
  */
 static void
@@ -470,7 +467,7 @@ emit_html_rules(GVJ_t * job, htmlcell_t * cp, htmlenv_t * env, char *color, html
     pts.UR.y += pos.y;
 
     //Determine vertical line coordinate and length
-    if ((cp->ruled & HTML_VRULE) && cp->col + cp->colspan < cp->parent->column_count) {
+    if (cp->vruled && cp->col + cp->colspan < cp->parent->column_count) {
 	if (cp->row == 0) {	// first row
 	    // extend to center of table border and add half cell spacing
 	    base = cp->parent->data.border + cp->parent->data.space / 2;
@@ -488,7 +485,7 @@ emit_html_rules(GVJ_t * job, htmlcell_t * cp, htmlenv_t * env, char *color, html
 	doSide(job, rule_pt, 0, rule_length);
     }
     //Determine the horizontal coordinate and length
-    if ((cp->ruled & HTML_HRULE) && cp->row + cp->rowspan < cp->parent->row_count) {
+    if (cp->hruled && cp->row + cp->rowspan < cp->parent->row_count) {
 	if (cp->col == 0) {	// first column 
 	    // extend to center of table border and add half cell spacing
 	    base = cp->parent->data.border + cp->parent->data.space / 2;
@@ -526,7 +523,7 @@ static void emit_html_tbl(GVJ_t * job, htmltbl_t * tbl, htmlenv_t * env)
     static textfont_t savef;
     htmlmap_data_t saved;
     int anchor;			/* if true, we need to undo anchor settings. */
-    int doAnchor = (tbl->data.href || tbl->data.target);
+    const bool doAnchor = tbl->data.href || tbl->data.target || tbl->data.title;
     pointf AF[4];
 
     if (tbl->font)
@@ -542,20 +539,21 @@ static void emit_html_tbl(GVJ_t * job, htmltbl_t * tbl, htmlenv_t * env)
     else
 	anchor = 0;
 
-    if (!(tbl->data.style & INVISIBLE)) {
+    if (!tbl->data.style.invisible) {
 
 	/* Fill first */
 	if (tbl->data.bgcolor) {
-	    char *clrs[2];
+	    char *clrs[2] = {0};
 	    int filled =
 		setFill(job, tbl->data.bgcolor, tbl->data.gradientangle,
 			tbl->data.style, clrs);
-	    if (tbl->data.style & ROUNDED) {
+	    if (tbl->data.style.rounded) {
 		round_corners(job, mkPts(AF, pts, tbl->data.border), 4,
-			      ROUNDED, filled);
+		              (graphviz_polygon_style_t){.rounded = true}, filled);
 	    } else
 		gvrender_box(job, pts, filled);
 	    free(clrs[0]);
+	    free(clrs[1]);
 	}
 
 	while (*cells) {
@@ -571,7 +569,7 @@ static void emit_html_tbl(GVJ_t * job, htmltbl_t * tbl, htmlenv_t * env)
 	cells = tbl->u.n.cells;
 	gvrender_set_penwidth(job, 1.0);
 	while ((cp = *cells++)) {
-	    if (cp->ruled)
+	    if (cp->hruled || cp->vruled)
 		emit_html_rules(job, cp, env, tbl->data.pencolor, *cells);
 	}
 
@@ -592,8 +590,7 @@ static void emit_html_tbl(GVJ_t * job, htmltbl_t * tbl, htmlenv_t * env)
 	popFontInfo(env, &savef);
 }
 
-/* emit_html_img:
- * The image will be centered in the given box.
+/* The image will be centered in the given box.
  * Scaling is determined by either the image's scale attribute,
  * or the imagescale attribute of the graph object being drawn.
  */
@@ -629,7 +626,8 @@ static void emit_html_cell(GVJ_t * job, htmlcell_t * cp, htmlenv_t * env)
     htmlmap_data_t saved;
     boxf pts = cp->data.box;
     pointf pos = env->pos;
-    int inAnchor, doAnchor = (cp->data.href || cp->data.target);
+    int inAnchor;
+    const bool doAnchor = cp->data.href || cp->data.target || cp->data.title;
     pointf AF[4];
 
     pts.LL.x += pos.x;
@@ -642,15 +640,15 @@ static void emit_html_cell(GVJ_t * job, htmlcell_t * cp, htmlenv_t * env)
     else
 	inAnchor = 0;
 
-    if (!(cp->data.style & INVISIBLE)) {
+    if (!cp->data.style.invisible) {
 	if (cp->data.bgcolor) {
 	    char *clrs[2];
 	    int filled =
 		setFill(job, cp->data.bgcolor, cp->data.gradientangle,
 			cp->data.style, clrs);
-	    if (cp->data.style & ROUNDED) {
+	    if (cp->data.style.rounded) {
 		round_corners(job, mkPts(AF, pts, cp->data.border), 4,
-			      ROUNDED, filled);
+			      (graphviz_polygon_style_t){.rounded = true}, filled);
 	    } else
 		gvrender_box(job, pts, filled);
 	    free(clrs[0]);
@@ -676,8 +674,7 @@ static void emit_html_cell(GVJ_t * job, htmlcell_t * cp, htmlenv_t * env)
     }
 }
 
-/* allocObj:
- * Push new obj on stack to be used in common by all 
+/* Push new obj on stack to be used in common by all 
  * html elements with anchors.
  * This inherits the type, emit_state, and object of the
  * parent, as well as the url, explicit, target and tooltip.
@@ -704,6 +701,8 @@ static void allocObj(GVJ_t * job)
     case EDGE_OBJTYPE:
 	obj->u.e = parent->u.e;
 	break;
+    default:
+	UNREACHABLE();
     }
     obj->url = parent->url;
     obj->tooltip = parent->tooltip;
@@ -743,8 +742,6 @@ heightOfLbl (htmllabel_t * lp)
     return sz;
 }
 
-/* emit_html_label:
- */
 void emit_html_label(GVJ_t * job, htmllabel_t * lp, textlabel_t * tp)
 {
     htmlenv_t env;
@@ -839,17 +836,16 @@ static void free_html_cell(htmlcell_t * cp)
     free(cp);
 }
 
-/* free_html_tbl:
- * If tbl->n_rows is negative, table is in initial state from
+/* If tbl->row_count is `SIZE_MAX`, table is in initial state from
  * HTML parse, with data stored in u.p. Once run through processTbl,
- * data is stored in u.n and tbl->n_rows is > 0.
+ * data is stored in u.n and tbl->row_count is < `SIZE_MAX`.
  */
 static void free_html_tbl(htmltbl_t * tbl)
 {
     htmlcell_t **cells;
 
     if (tbl->row_count == SIZE_MAX) { // raw, parsed table
-	dtclose(tbl->u.p.rows);
+	rows_free(&tbl->u.p.rows);
     } else {
 	cells = tbl->u.n.cells;
 
@@ -877,7 +873,7 @@ void free_html_label(htmllabel_t * lp, int root)
 	free(lp);
 }
 
-static htmldata_t *portToTbl(htmltbl_t *, char *);	/* forward declaration */
+static htmldata_t *portToTbl(htmltbl_t *, char *);
 
 static htmldata_t *portToCell(htmlcell_t * cp, char *id)
 {
@@ -893,8 +889,7 @@ static htmldata_t *portToCell(htmlcell_t * cp, char *id)
     return rv;
 }
 
-/* portToTbl:
- * See if tp or any of its child cells has the given port id.
+/* See if tp or any of its child cells has the given port id.
  * If true, return corresponding box.
  */
 static htmldata_t *portToTbl(htmltbl_t * tp, char *id)
@@ -917,13 +912,12 @@ static htmldata_t *portToTbl(htmltbl_t * tp, char *id)
     return rv;
 }
 
-/* html_port:
- * See if edge port corresponds to part of the html node.
- * Assume pname != "".
+/* See if edge port corresponds to part of the html node.
  * If successful, return pointer to port's box.
  * Else return NULL.
  */
 boxf *html_port(node_t *n, char *pname, unsigned char *sides){
+    assert(pname != NULL && !streq(pname, ""));
     htmldata_t *tp;
     htmllabel_t *lbl = ND_label(n)->u.html;
     boxf *rv = NULL;
@@ -1076,12 +1070,9 @@ static int size_html_txt(GVC_t *gvc, htmltxt_t * ftxt, htmlenv_t * env)
     return 0;
 }
 
-/* forward declarion for recursive usage */
 static int size_html_tbl(graph_t * g, htmltbl_t * tbl, htmlcell_t * parent,
 			 htmlenv_t * env);
 
-/* size_html_img:
- */
 static int size_html_img(htmlimg_t * img, htmlenv_t * env)
 {
     box b;
@@ -1092,7 +1083,7 @@ static int size_html_img(htmlimg_t * img, htmlenv_t * env)
     if (b.UR.x == -1 && b.UR.y == -1) {
 	rv = 1;
 	b.UR.x = b.UR.y = 0;
-	agerr(AGERR, "No or improper image file=\"%s\"\n", img->src);
+	agerrorf("No or improper image file=\"%s\"\n", img->src);
     } else {
 	rv = 0;
 	GD_has_images(env->g) = true;
@@ -1102,8 +1093,6 @@ static int size_html_img(htmlimg_t * img, htmlenv_t * env)
     return rv;
 }
 
-/* size_html_cell:
- */
 static int
 size_html_cell(graph_t * g, htmlcell_t * cp, htmltbl_t * parent,
 	       htmlenv_t * env)
@@ -1146,13 +1135,13 @@ size_html_cell(graph_t * g, htmlcell_t * cp, htmltbl_t * parent,
     if (cp->data.flags & FIXED_FLAG) {
 	if (cp->data.width && cp->data.height) {
 	    if ((cp->data.width < sz.x || cp->data.height < sz.y) && cp->child.kind != HTML_IMAGE) {
-		agerr(AGWARN, "cell size too small for content\n");
+		agwarningf("cell size too small for content\n");
 		rv = 1;
 	    }
 	    sz.x = sz.y = 0;
 
 	} else {
-	    agerr(AGWARN,
+	    agwarningf(
 		  "fixed cell size with unspecified width or height\n");
 	    rv = 1;
 	}
@@ -1162,8 +1151,7 @@ size_html_cell(graph_t * g, htmlcell_t * cp, htmltbl_t * parent,
     return rv;
 }
 
-static int findCol(PointSet * ps, int row, int col, htmlcell_t * cellp)
-{
+static uint16_t findCol(PointSet *ps, int row, int col, htmlcell_t *cellp) {
     int notFound = 1;
     int lastc;
     int i, j, c;
@@ -1185,56 +1173,40 @@ static int findCol(PointSet * ps, int row, int col, htmlcell_t * cellp)
 	    addPS(ps, j, i);
 	}
     }
-    return col;
+    assert(col >= 0 && col <= UINT16_MAX);
+    return (uint16_t)col;
 }
 
-/* processTbl:
- * Convert parser representation of cells into final form.
+/* Convert parser representation of cells into final form.
  * Find column and row positions of cells.
  * Recursively size cells.
  * Return 1 if problem sizing a cell.
  */
 static int processTbl(graph_t * g, htmltbl_t * tbl, htmlenv_t * env)
 {
-    pitem *rp;
-    pitem *cp;
-    Dt_t *cdict;
-    htmlcell_t *cellp;
     htmlcell_t **cells;
-    Dt_t *rows = tbl->u.p.rows;
+    rows_t rows = tbl->u.p.rows;
     int rv = 0;
     size_t n_rows = 0;
     size_t n_cols = 0;
     PointSet *ps = newPS();
-    Dt_t *is = openIntSet();
+    bitarray_t is = bitarray_new((size_t)UINT16_MAX + 1);
 
-    rp = (pitem *) dtflatten(rows);
     size_t cnt = 0;
-    uint16_t r = 0;
-    while (rp) {
-	cdict = rp->u.rp;
-	cp = (pitem *) dtflatten(cdict);
-	while (cp) {
-	    cellp = cp->u.cp;
-	    cnt++;
-	    cp = (pitem *)dtlink(cdict, cp);
-	}
+    for (uint16_t r = 0; r < rows_size(&rows); ++r) {
+	row_t *rp = rows_get(&rows, r);
+	cnt += cells_size(&rp->rp);
 	if (rp->ruled) {
-	    addIntSet(is, r + 1);
+	    bitarray_set(&is, r + 1, true);
 	}
-	rp = (pitem *)dtlink(rows, rp);
-	r++;
     }
 
-    cells = tbl->u.n.cells = N_NEW(cnt + 1, htmlcell_t *);
-    rp = (pitem *) dtflatten(rows);
-    r = 0;
-    while (rp) {
-	cdict = rp->u.rp;
-	cp = (pitem *) dtflatten(cdict);
+    cells = tbl->u.n.cells = gv_calloc(cnt + 1, sizeof(htmlcell_t *));
+    for (uint16_t r = 0; r < rows_size(&rows); ++r) {
+	row_t *rp = rows_get(&rows, r);
 	uint16_t c = 0;
-	while (cp) {
-	    cellp = cp->u.cp;
+	for (size_t i = 0; i < cells_size(&rp->rp); ++i) {
+	    htmlcell_t *cellp = cells_get(&rp->rp, i);
 	    *cells++ = cellp;
 	    rv |= size_html_cell(g, cellp, tbl, env);
 	    c = findCol(ps, r, c, cellp);
@@ -1243,258 +1215,168 @@ static int processTbl(graph_t * g, htmltbl_t * tbl, htmlenv_t * env)
 	    c += cellp->colspan;
 	    n_cols = MAX(c, n_cols);
 	    n_rows = MAX(r + cellp->rowspan, n_rows);
-	    if (inIntSet(is, r + cellp->rowspan))
-		cellp->ruled |= HTML_HRULE;
-	    cp = (pitem *)dtlink(cdict, cp);
+	    if (bitarray_get(is, r + cellp->rowspan))
+		cellp->hruled = true;
 	}
-	rp = (pitem *)dtlink(rows, rp);
-	r++;
     }
     tbl->row_count = n_rows;
     tbl->column_count = n_cols;
-    dtclose(rows);
-    dtclose(is);
+    rows_free(&rows);
+    bitarray_reset(&is);
     freePS(ps);
     return rv;
 }
 
-/* Split size x over n pieces with spacing s.
- * We subtract s*(n-1) from x, divide by n and 
- * take the ceiling.
- */
-#define SPLIT(x,n,s) (((x) - ((s)-1)*((n)-1)) / (n))
+/// set the widths of HTML table cells
+///
+/// Graphviz HTML tables were implemented prior to HTML standardization, but
+/// the subsequent RFC 1942¹ provides some guidance on “Recommended Layout
+/// Algorithms.” Following this, the W3C gave a slightly clearer articulation of
+/// essentially the same guidance when specifying CSS.² Many features discussed
+/// in this algorithm are not present in Graphviz’ variant of HTML (e.g. widths
+/// as percentages) or are not relevant (e.g. the derivation of _maximum_ column
+/// widths seems only relevant if you are laying out a table within another
+/// elastically sized container like a browser window), but the algorithm is
+/// useful nonetheless. This function implements an adapted version of the CSS
+/// specification algorithm. Comments that include quotes are quoting the CSS
+/// specification.
+///
+/// ¹ https://www.ietf.org/rfc/rfc1942.txt
+/// ² https://www.w3.org/TR/CSS2/tables.html#width-layout
+///
+/// @param table Table to update
+static void set_cell_widths(htmltbl_t *table) {
 
-/* sizeLinearArray:
- * Determine sizes of rows and columns. The size of a column is the
- * maximum width of any cell in it. Similarly for rows.
- * A cell spanning columns contributes proportionately to each column
- * it is in.
- */
-static void sizeLinearArray(htmltbl_t * tbl)
-{
-    htmlcell_t *cp;
-    htmlcell_t **cells;
-    int i;
+  // `processTbl` has already done step 1, “1. Calculate the minimum content
+  // width (MCW) of each cell”, and stored it in `.data.box.UR.x`.
 
-    tbl->heights = N_NEW(tbl->row_count + 1, double);
-    tbl->widths = N_NEW(tbl->column_count + 1, double);
+  // Allocate space for minimum column widths. Note that we add an extra entry
+  // to allow later code to make references like `table->widths[col + colspan]`.
+  assert(table->widths == NULL && "table widths computed twice");
+  table->widths = gv_calloc(table->column_count + 1, sizeof(double));
 
-    for (cells = tbl->u.n.cells; *cells; cells++) {
-	cp = *cells;
-	double ht;
-	if (cp->rowspan == 1)
-	    ht = cp->data.box.UR.y;
-	else {
-	    ht = SPLIT(cp->data.box.UR.y, cp->rowspan, tbl->data.space);
-	    ht = fmax(ht, 1);
-	}
-	double wd;
-	if (cp->colspan == 1)
-	    wd = cp->data.box.UR.x;
-	else {
-	    wd = SPLIT(cp->data.box.UR.x, cp->colspan, tbl->data.space);
-	    wd = fmax(wd, 1);
-	}
-	for (i = cp->row; i < cp->row + cp->rowspan; i++) {
-	    tbl->heights[i] = fmax(tbl->heights[i], ht);
-	}
-	for (i = cp->col; i < cp->col + cp->colspan; i++) {
-	    tbl->widths[i] = fmax(tbl->widths[i], wd);
-	}
+  // “2. For each column, determine a … minimum column width from the cells that
+  // span only that column. The minimum is that required by the cell with the
+  // largest minimum cell width …”. Note that this loop and the following one
+  // could be fused, but this would make the implementation harder to relate
+  // back to the specification.
+  for (htmlcell_t **i = table->u.n.cells; *i != NULL; ++i) {
+    const htmlcell_t cell = **i;
+    if (cell.colspan > 1) {
+      continue;
     }
+    assert(cell.col < table->column_count && "out of range cell");
+    table->widths[cell.col] = fmax(table->widths[cell.col], cell.data.box.UR.x);
+  }
+
+  // “3. For each cell that spans more than one column, increase the minimum
+  // widths of the columns it spans so that together, they are at least as wide
+  // as the cell. … If possible, widen all spanned columns by approximately the
+  // same amount.”
+  for (htmlcell_t **i = table->u.n.cells; *i != NULL; ++i) {
+    const htmlcell_t cell = **i;
+    if (cell.colspan == 1) {
+      continue;
+    }
+
+    // what is the current width of this cell’s column(s)’ span?
+    double span_width = 0;
+    assert(cell.col + cell.colspan <= table->column_count &&
+           "cell spans wider than containing table");
+    for (size_t j = 0; j < cell.colspan; ++j) {
+      span_width += table->widths[cell.col + j];
+    }
+
+    // if it is wider than its span (including cell spacing), widen the span
+    const double spacing = (cell.colspan - 1) * table->data.space;
+    if (span_width + spacing < cell.data.box.UR.x) {
+      const double widen_by =
+          (cell.data.box.UR.x - spacing - span_width) / cell.colspan;
+      for (size_t j = 0; j < cell.colspan; ++j) {
+        table->widths[cell.col + j] += widen_by;
+      }
+    }
+  }
+
+  // take the minimum width for each column and apply it to its contained cells
+  for (htmlcell_t **i = table->u.n.cells; *i != NULL; ++i) {
+    htmlcell_t *cell = *i;
+
+    // what is the current width of this cell’s column(s)’ span?
+    double min_width = 0;
+    assert(cell->col + cell->colspan <= table->column_count &&
+           "cell spans wider than containing table");
+    for (size_t j = 0; j < cell->colspan; ++j) {
+      min_width += table->widths[cell->col + j];
+    }
+
+    // widen the cell if necessary
+    const double spacing = (cell->colspan - 1) * table->data.space;
+    cell->data.box.UR.x = fmax(cell->data.box.UR.x, min_width + spacing);
+  }
 }
 
-/* closeGraphs:
- * Clean up graphs made for setting column and row widths.
- */
-static void closeGraphs(graph_t * rowg, graph_t * colg)
-{
-    node_t *n;
-    for (n = GD_nlist(colg); n; n = ND_next(n)) {
-	free_list(ND_in(n));
-	free_list(ND_out(n));
+/// set the heights of HTML table cells
+///
+/// This recapitulates the logic of `set_cell_widths` on rows.
+///
+/// @param table Table to update
+static void set_cell_heights(htmltbl_t *table) {
+
+  assert(table->heights == NULL && "table heights computed twice");
+  table->heights = gv_calloc(table->row_count + 1, sizeof(double));
+
+  for (htmlcell_t **i = table->u.n.cells; *i != NULL; ++i) {
+    const htmlcell_t cell = **i;
+    if (cell.rowspan > 1) {
+      continue;
+    }
+    assert(cell.row < table->row_count && "out of range cell");
+    table->heights[cell.row] =
+        fmax(table->heights[cell.row], cell.data.box.UR.y);
+  }
+
+  for (htmlcell_t **i = table->u.n.cells; *i != NULL; ++i) {
+    const htmlcell_t cell = **i;
+    if (cell.rowspan == 1) {
+      continue;
     }
 
-    agclose(rowg);
-    agclose(colg);
+    double span_height = 0;
+    assert(cell.row + cell.rowspan <= table->row_count &&
+           "cell spans higher than containing table");
+    for (size_t j = 0; j < cell.rowspan; ++j) {
+      span_height += table->heights[cell.row + j];
+    }
+
+    const double spacing = (cell.rowspan - 1) * table->data.space;
+    if (span_height + spacing < cell.data.box.UR.y) {
+      const double heighten_by =
+          (cell.data.box.UR.y - spacing - span_height) / cell.rowspan;
+      for (size_t j = 0; j < cell.rowspan; ++j) {
+        table->heights[cell.row + j] += heighten_by;
+      }
+    }
+  }
+
+  for (htmlcell_t **i = table->u.n.cells; *i != NULL; ++i) {
+    htmlcell_t *cell = *i;
+
+    double min_height = 0;
+    assert(cell->row + cell->rowspan <= table->row_count &&
+           "cell spans higher than containing table");
+    for (size_t j = 0; j < cell->rowspan; ++j) {
+      min_height += table->heights[cell->row + j];
+    }
+
+    const double spacing = (cell->rowspan - 1) * table->data.space;
+    cell->data.box.UR.y = fmax(cell->data.box.UR.y, min_height + spacing);
+  }
 }
 
-/* checkChain:
- * For each pair of nodes in the node list, add an edge if none exists.
- * Assumes node list has nodes ordered correctly.
- */
-static void checkChain(graph_t * g)
-{
-    node_t *t;
-    node_t *h;
-    edge_t *e;
-    t = GD_nlist(g);
-    for (h = ND_next(t); h; h = ND_next(h)) {
-	if (!agfindedge(g, t, h)) {
-	    e = agedge(g, t, h, NULL, 1);
-	    agbindrec(e, "Agedgeinfo_t", sizeof(Agedgeinfo_t), true);
-	    ED_minlen(e) = 0;
-	    elist_append(e, ND_out(t));
-	    elist_append(e, ND_in(h));
-	}
-	t = h;
-    }
-}
+static void pos_html_tbl(htmltbl_t *, boxf, unsigned char);
 
-/* checkEdge:
- * Check for edge in g. If it exists, set its minlen to max of sz and
- * current minlen. Else, create it and set minlen to sz.
- */
-static void
-checkEdge (graph_t* g, node_t* t, node_t* h, int sz)
-{
-    edge_t* e;
-
-    e = agfindedge (g, t, h);
-    if (e)
-	ED_minlen(e) = MAX(ED_minlen(e), sz);
-    else {
-	e = agedge(g, t, h, NULL, 1);
-	agbindrec(e, "Agedgeinfo_t", sizeof(Agedgeinfo_t), true);
-	ED_minlen(e) = sz;
-	elist_append(e, ND_out(t));
-	elist_append(e, ND_in(h));
-    }
-}
-
-/* makeGraphs:
- * Generate dags modeling the row and column constraints.
- * If the table has column_count columns, we create the graph
- *  0 -> 1 -> 2 -> ... -> column_count
- * and if a cell starts in column c with span colspan, with
- * width w, we add the edge c -> c+colspan [minlen = w].
- * Ditto for rows.
- *
- */
-static void makeGraphs(htmltbl_t * tbl, graph_t * rowg, graph_t * colg)
-{
-    htmlcell_t *cp;
-    htmlcell_t **cells;
-    node_t *t;
-    node_t *lastn;
-    node_t *h;
-    agxbuf value_buffer = {0};
-
-    lastn = NULL;
-    for (size_t i = 0; i <= tbl->column_count; i++) {
-	agxbprint(&value_buffer, "%" PRISIZE_T, i);
-	t = agnode(colg, agxbuse(&value_buffer), 1);
-	agbindrec(t, "Agnodeinfo_t", sizeof(Agnodeinfo_t), true);
-	alloc_elist(tbl->row_count, ND_in(t));
-	alloc_elist(tbl->row_count, ND_out(t));
-	if (lastn) {
-	    ND_next(lastn) = t;
-	    lastn = t;
-	} else {
-	    lastn = GD_nlist(colg) = t;
-	}
-    }
-    lastn = NULL;
-    for (size_t i = 0; i <= tbl->row_count; i++) {
-	agxbprint(&value_buffer, "%" PRISIZE_T, i);
-	t = agnode(rowg, agxbuse(&value_buffer), 1);
-	agbindrec(t, "Agnodeinfo_t", sizeof(Agnodeinfo_t), true);
-	alloc_elist(tbl->column_count, ND_in(t));
-	alloc_elist(tbl->column_count, ND_out(t));
-	if (lastn) {
-	    ND_next(lastn) = t;
-	    lastn = t;
-	} else {
-	    lastn = GD_nlist(rowg) = t;
-	}
-    }
-
-    for (cells = tbl->u.n.cells; *cells; cells++) {
-	cp = *cells;
-	agxbprint(&value_buffer, "%" PRIu16, cp->col);
-	t = agfindnode(colg, agxbuse(&value_buffer));
-	agxbprint(&value_buffer, "%d", cp->col + cp->colspan);
-	h = agfindnode(colg, agxbuse(&value_buffer));
-	checkEdge (colg, t, h, cp->data.box.UR.x);
-
-	agxbprint(&value_buffer, "%" PRIu16, cp->row);
-	t = agfindnode(rowg, agxbuse(&value_buffer));
-	agxbprint(&value_buffer, "%d", cp->row + cp->rowspan);
-	h = agfindnode(rowg, agxbuse(&value_buffer));
-	checkEdge (rowg, t, h, cp->data.box.UR.y);
-    }
-
-    agxbfree(&value_buffer);
-
-    /* Make sure that 0 <= 1 <= 2 ...k. This implies graph connected. */
-    checkChain(colg);
-    checkChain(rowg);
-}
-
-/* setSizes:
- * Use rankings to determine cell dimensions. The rank values
- * give the coordinate, so to get the width/height, we have
- * to subtract the previous value.
- */
-static void setSizes(htmltbl_t * tbl, graph_t * rowg, graph_t * colg)
-{
-    int i;
-    node_t *n;
-    int prev;
-
-    prev = 0;
-    n = GD_nlist(rowg);
-    for (i = 0, n = ND_next(n); n; i++, n = ND_next(n)) {
-	tbl->heights[i] = ND_rank(n) - prev;
-	prev = ND_rank(n);
-    }
-    prev = 0;
-    n = GD_nlist(colg);
-    for (i = 0, n = ND_next(n); n; i++, n = ND_next(n)) {
-	tbl->widths[i] = ND_rank(n) - prev;
-	prev = ND_rank(n);
-    }
-
-}
-
-/* sizeArray:
- * Set column and row sizes. Optimize for minimum width and
- * height. Where there is slack, try to distribute evenly.
- * We do this by encoding cells as edges with min length is
- * a dag on a chain. We then run network simplex, using
- * LR_balance.
- */
-static void sizeArray(htmltbl_t * tbl)
-{
-    graph_t *rowg;
-    graph_t *colg;
-    Agdesc_t dir = Agstrictdirected;
-
-    /* Do the 1D cases by hand */
-    if (tbl->row_count == 1 || tbl->column_count == 1) {
-	sizeLinearArray(tbl);
-	return;
-    }
-
-    tbl->heights = N_NEW(tbl->row_count + 1, double);
-    tbl->widths = N_NEW(tbl->column_count + 1, double);
-
-    rowg = agopen("rowg", dir, NULL);
-    colg = agopen("colg", dir, NULL);
-    /* Only need GD_nlist */
-    agbindrec(rowg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);	// graph custom data
-    agbindrec(colg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);	// graph custom data
-    makeGraphs(tbl, rowg, colg);
-    rank(rowg, 2, INT_MAX);
-    rank(colg, 2, INT_MAX);
-    setSizes(tbl, rowg, colg);
-    closeGraphs(rowg, colg);
-}
-
-static void pos_html_tbl(htmltbl_t *, boxf, int);	/* forward declaration */
-
-/* pos_html_img:
- * Place image in cell
+/* Place image in cell
  * storing allowed space handed by parent cell.
  * How this space is used is handled in emit_html_img.
  */
@@ -1503,9 +1385,7 @@ static void pos_html_img(htmlimg_t * cp, boxf pos)
     cp->box = pos;
 }
 
-/* pos_html_txt:
- * Set default alignment.
- */
+/// Set default alignment.
 static void pos_html_txt(htmltxt_t * ftxt, char c)
 {
     for (size_t i = 0; i < ftxt->nspans; i++) {
@@ -1514,10 +1394,7 @@ static void pos_html_txt(htmltxt_t * ftxt, char c)
     }
 }
 
-/* pos_html_cell:
- */
-static void pos_html_cell(htmlcell_t * cp, boxf pos, int sides)
-{
+static void pos_html_cell(htmlcell_t *cp, boxf pos, unsigned char sides) {
     double delx, dely;
     pointf oldsz;
     boxf cbox;
@@ -1584,6 +1461,8 @@ static void pos_html_cell(htmlcell_t * cp, boxf pos, int sides)
 	    case HALIGN_RIGHT:
 		cbox.LL.x += delx;
 		break;
+	    default:
+		break;
 	    }
 	}
 
@@ -1596,6 +1475,8 @@ static void pos_html_cell(htmlcell_t * cp, boxf pos, int sides)
 	    case VALIGN_TOP:
 		cbox.LL.y += dely;
 		break;
+	    default:
+		break;
 	    }
 	}
 	pos_html_img(cp->child.u.img, cbox);
@@ -1657,14 +1538,12 @@ static void pos_html_cell(htmlcell_t * cp, boxf pos, int sides)
     }
 }
 
-/* pos_html_tbl:
- * Position table given its box, then calculate
+/* Position table given its box, then calculate
  * the position of each cell. In addition, set the sides
  * attribute indicating which external sides of the node
  * are accessible to the table.
  */
-static void pos_html_tbl(htmltbl_t * tbl, boxf pos, int sides)
-{
+static void pos_html_tbl(htmltbl_t *tbl, boxf pos, unsigned char sides) {
     int plus;
     htmlcell_t **cells = tbl->u.n.cells;
     htmlcell_t *cp;
@@ -1736,7 +1615,7 @@ static void pos_html_tbl(htmltbl_t * tbl, boxf pos, int sides)
     }
 
     while ((cp = *cells++)) {
-	int mask = 0;
+	unsigned char mask = 0;
 	if (sides) {
 	    if (cp->col == 0)
 		mask |= LEFT;
@@ -1758,8 +1637,7 @@ static void pos_html_tbl(htmltbl_t * tbl, boxf pos, int sides)
     tbl->data.box = pos;
 }
 
-/* size_html_tbl:
- * Determine the size of a table by first determining the
+/* Determine the size of a table by first determining the
  * size of each cell.
  */
 static int
@@ -1782,7 +1660,8 @@ size_html_tbl(graph_t * g, htmltbl_t * tbl, htmlcell_t * parent,
 	tbl->data.border = DEFAULT_BORDER;
     }
 
-    sizeArray(tbl);
+    set_cell_widths(tbl);
+    set_cell_heights(tbl);
 
     assert(tbl->column_count <= DBL_MAX);
     double wd = ((double)tbl->column_count + 1) * tbl->data.space + 2 * tbl->data.border;
@@ -1796,13 +1675,13 @@ size_html_tbl(graph_t * g, htmltbl_t * tbl, htmlcell_t * parent,
     if (tbl->data.flags & FIXED_FLAG) {
 	if (tbl->data.width && tbl->data.height) {
 	    if (tbl->data.width < wd || tbl->data.height < ht) {
-		agerr(AGWARN, "table size too small for content\n");
+		agwarningf("table size too small for content\n");
 		rv = 1;
 	    }
 	    wd = 0;
 	    ht = 0;
 	} else {
-	    agerr(AGWARN,
+	    agwarningf(
 		  "fixed table size with unspecified width or height\n");
 	    rv = 1;
 	}
@@ -1970,9 +1849,7 @@ static char *getPenColor(void *obj)
 	return NULL;
 }
 
-/* make_html_label:
- * Return non-zero if problem parsing HTML. In this case, use object name.
- */
+/// Return non-zero if problem parsing HTML. In this case, use object name.
 int make_html_label(void *obj, textlabel_t * lp)
 {
     int rv;
@@ -2003,8 +1880,10 @@ int make_html_label(void *obj, textlabel_t * lp)
     lbl = parseHTML(lp->text, &rv, &env);
     if (!lbl) {
 	if (rv == 3) {
-	    // fatal error
-	    graphviz_exit(EXIT_FAILURE);
+	    // fatal error; `parseHTML` will have printed detail of it
+	    lp->html = false;
+	    lp->text = gv_strdup(lp->text);
+	    return rv;
 	}
 	/* Parse of label failed; revert to simple text label */
 	agxbuf xb = {0};
