@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -12,13 +12,12 @@
 #include <string.h>
 #include <math.h>
 #include <assert.h>
-#include <cgraph/alloc.h>
 #include <common/arith.h>
 #include <limits.h>
 #include <sparse/SparseMatrix.h>
-#include <sparse/BinaryHeap.h>
 #include <stddef.h>
 #include <stdbool.h>
+#include <util/alloc.h>
 
 static size_t size_of_matrix_type(int type){
   size_t size = 0;
@@ -58,7 +57,7 @@ SparseMatrix SparseMatrix_make_undirected(SparseMatrix A){
   /* make it strictly low diag only, and set flag to undirected */
   SparseMatrix B;
   B = SparseMatrix_symmetrize(A, false);
-  SparseMatrix_set_undirected(B);
+  B->is_undirected = true;
   return SparseMatrix_remove_upper(B);
 }
 SparseMatrix SparseMatrix_transpose(SparseMatrix A){
@@ -151,44 +150,29 @@ SparseMatrix SparseMatrix_symmetrize(SparseMatrix A,
   if (!B) return NULL;
   A = SparseMatrix_add(A, B);
   SparseMatrix_delete(B);
-  SparseMatrix_set_symmetric(A);
-  SparseMatrix_set_pattern_symmetric(A);
+  A->is_symmetric = true;
+  A->is_pattern_symmetric = true;
   return A;
 }
 
-SparseMatrix SparseMatrix_symmetrize_nodiag(SparseMatrix A){
-  SparseMatrix B;
-  if (SparseMatrix_is_symmetric(A, false)) {
-    B = SparseMatrix_copy(A);
-    return SparseMatrix_remove_diagonal(B);
-  }
-  B = SparseMatrix_transpose(A);
-  if (!B) return NULL;
-  A = SparseMatrix_add(A, B);
-  SparseMatrix_delete(B);
-  SparseMatrix_set_symmetric(A);
-  SparseMatrix_set_pattern_symmetric(A);
-  return SparseMatrix_remove_diagonal(A);
-}
-
-int SparseMatrix_is_symmetric(SparseMatrix A, bool test_pattern_symmetry_only){
-  if (!A) return FALSE;
+bool SparseMatrix_is_symmetric(SparseMatrix A, bool test_pattern_symmetry_only) {
+  if (!A) return false;
 
   /* assume no repeated entries! */
   SparseMatrix B;
   int *ia, *ja, *ib, *jb, type, m;
   int *mask;
-  int res = FALSE;
+  bool res = false;
   int i, j;
   assert(A->format == FORMAT_CSR);/* only implemented for CSR right now */
 
-  if (SparseMatrix_known_symmetric(A)) return TRUE;
-  if (test_pattern_symmetry_only && SparseMatrix_known_strucural_symmetric(A)) return TRUE;
+  if (A->is_symmetric) return true;
+  if (test_pattern_symmetry_only && A->is_pattern_symmetric) return true;
 
-  if (A->m != A->n) return FALSE;
+  if (A->m != A->n) return false;
 
   B = SparseMatrix_transpose(A);
-  if (!B) return FALSE;
+  if (!B) return false;
 
   ia = A->ia;
   ja = A->ja;
@@ -218,7 +202,7 @@ int SparseMatrix_is_symmetric(SparseMatrix A, bool test_pattern_symmetry_only){
 	if (fabs(b[j] - a[mask[jb[j]]]) > SYMMETRY_EPSILON) goto RETURN;
       }
     }
-    res = TRUE;
+    res = true;
     break;
   }
   case MATRIX_TYPE_COMPLEX:{
@@ -237,7 +221,7 @@ int SparseMatrix_is_symmetric(SparseMatrix A, bool test_pattern_symmetry_only){
 	if (fabs(b[2*j+1] - a[2*mask[jb[j]]+1]) > SYMMETRY_EPSILON) goto RETURN;
       }
     }
-    res = TRUE;
+    res = true;
     break;
   }
   case MATRIX_TYPE_INTEGER:{
@@ -254,7 +238,7 @@ int SparseMatrix_is_symmetric(SparseMatrix A, bool test_pattern_symmetry_only){
 	if (bi[j] != ai[mask[jb[j]]]) goto RETURN;
       }
     }
-    res = TRUE;
+    res = true;
     break;
   }
   case MATRIX_TYPE_PATTERN:
@@ -266,7 +250,7 @@ int SparseMatrix_is_symmetric(SparseMatrix A, bool test_pattern_symmetry_only){
 	if (mask[jb[j]] < ia[i]) goto RETURN;
       }
     }
-    res = TRUE;
+    res = true;
     break;
   case MATRIX_TYPE_UNKNOWN:
     goto RETURN;
@@ -276,12 +260,10 @@ int SparseMatrix_is_symmetric(SparseMatrix A, bool test_pattern_symmetry_only){
     break;
   }
 
-  if (test_pattern_symmetry_only){
-    SparseMatrix_set_pattern_symmetric(A);
-  } else {
-    SparseMatrix_set_symmetric(A);
-    SparseMatrix_set_pattern_symmetric(A);
+  if (!test_pattern_symmetry_only) {
+    A->is_symmetric = true;
   }
+  A->is_pattern_symmetric = true;
  RETURN:
   free(mask);
 
@@ -301,7 +283,6 @@ static SparseMatrix SparseMatrix_init(int m, int n, int type, size_t sz, int for
   case FORMAT_COORD:
     A->ia = NULL;
     break;
-  case FORMAT_CSC:
   case FORMAT_CSR:
   default:
     A->ia = gv_calloc((size_t)(m + 1), sizeof(int));
@@ -309,7 +290,6 @@ static SparseMatrix SparseMatrix_init(int m, int n, int type, size_t sz, int for
   A->ja = NULL;
   A->a = NULL;
   A->format = format;
-  A->property = 0;
   return A;
 }
 
@@ -325,7 +305,6 @@ static SparseMatrix SparseMatrix_alloc(SparseMatrix A, int nz){
     A->a = gv_calloc(nz_t, A->size);
     break;
   case FORMAT_CSR:
-  case FORMAT_CSC:
   default:
     A->ja = gv_calloc(nz_t, sizeof(int));
     if (A->size > 0 && nz_t > 0) {
@@ -354,7 +333,6 @@ static SparseMatrix SparseMatrix_realloc(SparseMatrix A, int nz){
     } 
     break;
   case FORMAT_CSR:
-  case FORMAT_CSC:
   default:
     A->ja = gv_recalloc(A->ja, A->nzmax, nz_t, sizeof(int));
     if (A->size > 0) {
@@ -404,142 +382,6 @@ void SparseMatrix_delete(SparseMatrix A){
   free(A->a);
   free(A);
 }
-static void SparseMatrix_print_csr(char *c, SparseMatrix A){
-  int *ia, *ja;
-  double *a;
-  int *ai;
-  int i, j, m = A->m;
-  
-  assert (A->format == FORMAT_CSR);
-  printf("%s\n SparseArray[{",c);
-  ia = A->ia;
-  ja = A->ja;
-  a = A->a;
-  switch (A->type){
-  case MATRIX_TYPE_REAL:
-    a = A->a;
-    for (i = 0; i < m; i++){
-      for (j = ia[i]; j < ia[i+1]; j++){
-	printf("{%d, %d}->%f",i+1, ja[j]+1, a[j]);
-	if (j != ia[m]-1) printf(",");
-      }
-    }
-    break;
-  case MATRIX_TYPE_COMPLEX:
-    a = A->a;
-    for (i = 0; i < m; i++){
-      for (j = ia[i]; j < ia[i+1]; j++){
-	printf("{%d, %d}->%f + %f I",i+1, ja[j]+1, a[2*j], a[2*j+1]);
- 	if (j != ia[m]-1) printf(",");
-     }
-    }
-    printf("\n");
-    break;
-  case MATRIX_TYPE_INTEGER:
-    ai = A->a;
-    for (i = 0; i < m; i++){
-      for (j = ia[i]; j < ia[i+1]; j++){
-	printf("{%d, %d}->%d",i+1, ja[j]+1, ai[j]);
- 	if (j != ia[m]-1) printf(",");
-     }
-    }
-    printf("\n");
-    break;
-  case MATRIX_TYPE_PATTERN:
-    for (i = 0; i < m; i++){
-      for (j = ia[i]; j < ia[i+1]; j++){
-	printf("{%d, %d}->_",i+1, ja[j]+1);
- 	if (j != ia[m]-1) printf(",");
-      }
-    }
-    printf("\n");
-    break;
-  case MATRIX_TYPE_UNKNOWN:
-    return;
-  default:
-    return;
-  }
-  printf("},{%d, %d}]\n", m, A->n);
-
-}
-
-
-
-static void SparseMatrix_print_coord(char *c, SparseMatrix A){
-  int *ia, *ja;
-  double *a;
-  int *ai;
-  int i, m = A->m;
-  
-  assert (A->format == FORMAT_COORD);
-  printf("%s\n SparseArray[{",c);
-  ia = A->ia;
-  ja = A->ja;
-  a = A->a;
-  switch (A->type){
-  case MATRIX_TYPE_REAL:
-    a = A->a;
-    for (i = 0; i < A->nz; i++){
-      printf("{%d, %d}->%f",ia[i]+1, ja[i]+1, a[i]);
-      if (i != A->nz - 1) printf(",");
-    }
-    printf("\n");
-    break;
-  case MATRIX_TYPE_COMPLEX:
-    a = A->a;
-    for (i = 0; i < A->nz; i++){
-      printf("{%d, %d}->%f + %f I",ia[i]+1, ja[i]+1, a[2*i], a[2*i+1]);
-      if (i != A->nz - 1) printf(",");
-    }
-    printf("\n");
-    break;
-  case MATRIX_TYPE_INTEGER:
-    ai = A->a;
-    for (i = 0; i < A->nz; i++){
-      printf("{%d, %d}->%d",ia[i]+1, ja[i]+1, ai[i]);
-      if (i != A->nz) printf(",");
-    }
-    printf("\n");
-    break;
-  case MATRIX_TYPE_PATTERN:
-    for (i = 0; i < A->nz; i++){
-      printf("{%d, %d}->_",ia[i]+1, ja[i]+1);
-      if (i != A->nz - 1) printf(",");
-    }
-    printf("\n");
-    break;
-  case MATRIX_TYPE_UNKNOWN:
-    return;
-  default:
-    return;
-  }
-  printf("},{%d, %d}]\n", m, A->n);
-
-}
-
-
-
-
-void SparseMatrix_print(char *c, SparseMatrix A){
-  switch (A->format){
-  case FORMAT_CSR:
-    SparseMatrix_print_csr(c, A);
-    break;
-  case FORMAT_CSC:
-    assert(0);/* not implemented yet...
-		 SparseMatrix_print_csc(c, A);*/
-    break;
-  case FORMAT_COORD:
-    SparseMatrix_print_coord(c, A);
-    break;
-  default:
-    assert(0);
-  }
-}
-
-
-
-
 
 static void SparseMatrix_export_csr(FILE *f, SparseMatrix A){
   int *ia, *ja;
@@ -678,10 +520,6 @@ void SparseMatrix_export(FILE *f, SparseMatrix A){
   case FORMAT_CSR:
     SparseMatrix_export_csr(f, A);
     break;
-  case FORMAT_CSC:
-    assert(0);/* not implemented yet...
-		 SparseMatrix_print_csc(c, A);*/
-    break;
   case FORMAT_COORD:
     SparseMatrix_export_coord(f, A);
     break;
@@ -991,10 +829,12 @@ SparseMatrix SparseMatrix_add(SparseMatrix A, SparseMatrix B){
   return C;
 }
 
-static void SparseMatrix_multiply_dense1(SparseMatrix A, double *v, double **res, int dim){
-  /* A v where v a dense matrix of second dimension dim. Real only for now. */
+void SparseMatrix_multiply_dense(SparseMatrix A, const double *v, double *res,
+                                 int dim) {
+  // A × V, with A dimension m × n, with V a dense matrix of dimension n × dim.
+  // v[i×dim×j] gives V[i,j]. Result of dimension m × dim. Real only for now.
   int i, j, k, *ia, *ja, m;
-  double *a, *u;
+  double *a;
 
   assert(A->format == FORMAT_CSR);
   assert(A->type == MATRIX_TYPE_REAL);
@@ -1003,24 +843,13 @@ static void SparseMatrix_multiply_dense1(SparseMatrix A, double *v, double **res
   ia = A->ia;
   ja = A->ja;
   m = A->m;
-  u = *res;
 
-  if (!u) u = gv_calloc((size_t)m * (size_t)dim, sizeof(double));
   for (i = 0; i < m; i++){
-    for (k = 0; k < dim; k++) u[i*dim+k] = 0.;
+    for (k = 0; k < dim; k++) res[i * dim + k] = 0;
     for (j = ia[i]; j < ia[i+1]; j++){
-      for (k = 0; k < dim; k++) u[i*dim+k] += a[j]*v[ja[j]*dim+k];
+      for (k = 0; k < dim; k++) res[i * dim + k] += a[j] * v[ja[j] *dim + k];
     }
   }
-
-  *res = u;
-}
-
-void SparseMatrix_multiply_dense(SparseMatrix A, double *v, double **res, int dim){
-  /* A * V, with A dimension m x n, with V of dimension n x dim. v[i*dim+j] gives V[i,j]. Result of dimension m x dim
- */
-
-  SparseMatrix_multiply_dense1(A, v, res, dim);
 }
 
 void SparseMatrix_multiply_vector(SparseMatrix A, double *v, double **res) {
@@ -1121,6 +950,7 @@ SparseMatrix SparseMatrix_multiply(SparseMatrix A, SparseMatrix B){
 #ifdef DEBUG_PRINT
 	    fprintf(stderr,"overflow in SparseMatrix_multiply !!!\n");
 #endif
+	    free(mask);
 	    return NULL;
 	  }
 	  nz++;
@@ -1431,12 +1261,9 @@ SparseMatrix SparseMatrix_sum_repeat_entries(SparseMatrix A){
       }
     }
     break;
-  case MATRIX_TYPE_UNKNOWN:
-    return NULL;
-    break;
   default:
+    free(mask);
     return NULL;
-    break;
   }
   A->nz = nz;
   free(mask);
@@ -1444,7 +1271,7 @@ SparseMatrix SparseMatrix_sum_repeat_entries(SparseMatrix A){
 }
 
 SparseMatrix SparseMatrix_coordinate_form_add_entry(SparseMatrix A, int irn,
-                                                    int jcn, void *val) {
+                                                    int jcn, const void *val) {
   int nz, nzmax;
 
   static const int nentries = 1;
@@ -1623,8 +1450,8 @@ SparseMatrix SparseMatrix_remove_upper(SparseMatrix A){/* remove diag and upper
     return NULL;
   }
 
-  clear_flag(A->property, MATRIX_PATTERN_SYMMETRIC);
-  clear_flag(A->property, MATRIX_SYMMETRIC);
+  A->is_pattern_symmetric = false;
+  A->is_symmetric = false;
   return A;
 }
 
@@ -1745,7 +1572,9 @@ SparseMatrix SparseMatrix_copy(SparseMatrix A){
     memcpy(B->ja, A->ja, sizeof(int)*((size_t)(A->ia[A->m])));
   }
   if (A->a) memcpy(B->a, A->a, A->size*((size_t)A->nz));
-  B->property = A->property;
+  B->is_pattern_symmetric = A->is_pattern_symmetric;
+  B->is_symmetric = A->is_symmetric;
+  B->is_undirected = A->is_undirected;
   B->nz = A->nz;
   return B;
 }
@@ -1762,11 +1591,12 @@ bool SparseMatrix_has_diagonal(SparseMatrix A) {
   return false;
 }
 
-static void SparseMatrix_level_sets_internal(int khops, SparseMatrix A, int root, int *nlevel, int **levelset_ptr, int **levelset, int **mask, int reinitialize_mask){
+static void SparseMatrix_level_sets(SparseMatrix A, int root, int *nlevel,
+                                    int **levelset_ptr, int **levelset,
+                                    int **mask, bool reinitialize_mask) {
   /* mask is assumed to be initialized to negative if provided.
      . On exit, mask = levels for visited nodes (1 for root, 2 for its neighbors, etc), 
-     . unless reinitialize_mask = TRUE, in which case mask = -1.
-     khops: max number of hops allowed. If khops < 0, no limit is applied.
+     . unless reinitialize_mask is true, in which case mask = -1.
      A: the graph, undirected
      root: starting node
      nlevel: max distance to root from any node (in the connected comp)
@@ -1791,7 +1621,7 @@ static void SparseMatrix_level_sets_internal(int khops, SparseMatrix A, int root
   *nlevel = 1;
   nz = 1;
   sta = 0; sto = 1;
-  while (sto > sta && (khops < 0 || *nlevel <= khops)){
+  while (sto > sta){
     for (i = sta; i < sto; i++){
       ii = (*levelset)[i];
       for (j = ia[ii]; j < ia[ii+1]; j++){
@@ -1806,21 +1636,12 @@ static void SparseMatrix_level_sets_internal(int khops, SparseMatrix A, int root
     sta = sto;
     sto = nz;
   }
-  if (khops < 0 || *nlevel <= khops){
-    (*nlevel)--;
-  }
+  (*nlevel)--;
   if (reinitialize_mask) for (i = 0; i < (*levelset_ptr)[*nlevel]; i++) (*mask)[(*levelset)[i]] = UNMASKED;
 }
 
-void SparseMatrix_level_sets(SparseMatrix A, int root, int *nlevel, int **levelset_ptr, int **levelset, int **mask, int reinitialize_mask){
-
-  int khops = -1;
-
-  return SparseMatrix_level_sets_internal(khops, A, root, nlevel, levelset_ptr, levelset, mask, reinitialize_mask);
-
-}
-
-void SparseMatrix_weakly_connected_components(SparseMatrix A0, int *ncomp, int **comps, int **comps_ptr){
+int *SparseMatrix_weakly_connected_components(SparseMatrix A0, int *ncomp,
+                                              int **comps) {
   SparseMatrix A = A0;
   int *levelset_ptr = NULL, *levelset = NULL, *mask = NULL, nlevel;
   int m = A->m, i, nn;
@@ -1828,17 +1649,17 @@ void SparseMatrix_weakly_connected_components(SparseMatrix A0, int *ncomp, int *
   if (!SparseMatrix_is_symmetric(A, true)){
     A = SparseMatrix_symmetrize(A, true);
   }
-  if (!(*comps_ptr)) *comps_ptr = gv_calloc((size_t)(m + 1), sizeof(int));
+  int *comps_ptr = gv_calloc((size_t)(m + 1), sizeof(int));
 
   *ncomp = 0;
-  (*comps_ptr)[0] = 0;
+  comps_ptr[0] = 0;
   for (i = 0; i < m; i++){
     if (i == 0 || mask[i] < 0) {
-      SparseMatrix_level_sets(A, i, &nlevel, &levelset_ptr, &levelset, &mask, FALSE);
+      SparseMatrix_level_sets(A, i, &nlevel, &levelset_ptr, &levelset, &mask, false);
       if (i == 0) *comps = levelset;
       nn = levelset_ptr[nlevel];
       levelset += nn;
-      (*comps_ptr)[(*ncomp)+1] = (*comps_ptr)[(*ncomp)] + nn;
+      comps_ptr[(*ncomp)+1] = comps_ptr[(*ncomp)] + nn;
       (*ncomp)++;
     }
     
@@ -1847,143 +1668,7 @@ void SparseMatrix_weakly_connected_components(SparseMatrix A0, int *ncomp, int *
   free(levelset_ptr);
 
   free(mask);
-}
-
-
-
-struct nodedata_struct {
-  double dist;/* distance to root */
-  int id;/*node id */
-};
-typedef struct nodedata_struct* nodedata;
-
-static int cmp(void*i, void*j){
-  nodedata d1, d2;
-
-  d1 = i;
-  d2 = j;
-  if (d1->dist > d2->dist){
-    return 1;
-  }
-  if (d1->dist < d2->dist) {
-    return -1;
-  }
-  return 0;
-}
-
-static int Dijkstra_internal(SparseMatrix A, int root, double *dist, int *nlist, int *list, double *dist_max, int *mask){
-  /* Find the shortest path distance of all nodes to root. If khops >= 0, the shortest ath is of distance <= khops,
-
-     A: the nxn connectivity matrix. Entries are assumed to be nonnegative. Absolute value will be taken if 
-     .  entry value is negative.
-     dist: length n. On on exit contain the distance from root to every other node. dist[root] = 0. dist[i] = distance from root to node i.
-     .     if the graph is disconnected, unreachable node have a distance -1.
-     .     note: ||root - list[i]|| =!= dist[i] !!!, instead, ||root - list[i]|| == dist[list[i]]
-     nlist: number of nodes visited
-     list: length n. the list of node in order of their extraction from the heap. 
-     .     The distance from root to last in the list should be the maximum
-     dist_max: the maximum distance, should be realized at node list[nlist-1].
-     mask: if NULL, not used. Otherwise, only nodes i with mask[i] > 0 will be considered
-     return: 0 if every node is reachable. -1 if not */
-
-  int m = A->m, i, j, jj, *ia = A->ia, *ja = A->ja, heap_id;
-  BinaryHeap h;
-  double *a = NULL, *aa;
-  int *ai;
-  nodedata ndata, ndata_min;
-  enum {UNVISITED = -2, FINISHED = -1};
-  int *heap_ids; /* node ID to heap ID array. Initialised to UNVISITED. 
-		   Set to FINISHED after extracted as min from heap */
-  int found = 0;
-
-  assert(SparseMatrix_is_symmetric(A, true));
-
-  assert(m == A->n);
-
-  switch (A->type){
-  case MATRIX_TYPE_COMPLEX:
-    aa = A->a;
-    a = gv_calloc((size_t)A->nz, sizeof(double));
-    for (i = 0; i < A->nz; i++) a[i] = aa[i*2];
-    break;
-  case MATRIX_TYPE_REAL:
-    a = A->a;
-    break;
-  case MATRIX_TYPE_INTEGER:
-    ai = A->a;
-    a = gv_calloc((size_t)A->nz, sizeof(double));
-    for (i = 0; i < A->nz; i++) a[i] = (double) ai[i];
-    break;
-  case MATRIX_TYPE_PATTERN:
-    a = gv_calloc((size_t)A->nz, sizeof(double));
-    for (i = 0; i < A->nz; i++) a[i] = 1.;
-    break;
-  default:
-    assert(0);/* no such matrix type */
-  }
-
-  heap_ids = gv_calloc((size_t)m, sizeof(int));
-  for (i = 0; i < m; i++) {
-    dist[i] = -1;
-    heap_ids[i] = UNVISITED;
-  }
-
-  h = BinaryHeap_new(cmp);
-  assert(h);
-
-  /* add root as the first item in the heap */
-  ndata = gv_alloc(sizeof(struct nodedata_struct));
-  ndata->dist = 0;
-  ndata->id = root;
-  heap_ids[root] = BinaryHeap_insert(h, ndata);
-  
-  assert(heap_ids[root] >= 0);/* by design heap ID from BinaryHeap_insert >=0*/
-
-  while ((ndata_min = BinaryHeap_extract_min(h))){
-    i = ndata_min->id;
-    dist[i] = ndata_min->dist;
-    list[found++] = i;
-    heap_ids[i] = FINISHED;
-    //fprintf(stderr," =================\n min extracted is id=%d, dist=%f\n",i, ndata_min->dist);
-    for (j = ia[i]; j < ia[i+1]; j++){
-      jj = ja[j];
-      heap_id = heap_ids[jj];
-
-      if (jj == i || heap_id == FINISHED || (mask && mask[jj] < 0)) continue;
-
-      if (heap_id == UNVISITED){
-	ndata = gv_alloc(sizeof(struct nodedata_struct));
-	ndata->dist = fabs(a[j]) + ndata_min->dist;
-	ndata->id = jj;
-	heap_ids[jj] = BinaryHeap_insert(h, ndata);
-	//fprintf(stderr," set neighbor id=%d, dist=%f, hid = %d, a[%d]=%f, dist=%f\n",jj, ndata->dist, heap_ids[jj], jj, a[j], ndata->dist);
-
-      } else {
-	ndata = BinaryHeap_get_item(h, heap_id);
-	ndata->dist = MIN(ndata->dist, fabs(a[j]) + ndata_min->dist);
-	assert(ndata->id == jj);
-	BinaryHeap_reset(h, heap_id, ndata); 
-	//fprintf(stderr," reset neighbor id=%d, dist=%f, hid = %d, a[%d]=%f, dist=%f\n",jj, ndata->dist,heap_id, jj, a[j], ndata->dist);
-     }
-    }
-    free(ndata_min);
-  }
-  *nlist = found;
-  *dist_max = dist[i];
-
-
-  BinaryHeap_delete(h, free);
-  free(heap_ids);
-  if (a && a != A->a) free(a);
-  if (found == m || mask){
-    return 0;
-  } else {
-    return -1;
-  }
-}
-
-static int Dijkstra(SparseMatrix A, int root, double *dist, int *nlist, int *list, double *dist_max){
-  return Dijkstra_internal(A, root, dist, nlist, list, dist_max, NULL);
+  return comps_ptr;
 }
 
 void SparseMatrix_decompose_to_supervariables(SparseMatrix A, int *ncluster, int **cluster, int **clusterp){
@@ -2119,8 +1804,8 @@ SparseMatrix SparseMatrix_get_augmented(SparseMatrix A){
   }
 
   B = SparseMatrix_from_coordinate_arrays(nz, m + n, m + n, irn, jcn, val, type, A->size);
-  SparseMatrix_set_symmetric(B);
-  SparseMatrix_set_pattern_symmetric(B);
+  B->is_symmetric = true;
+  B->is_pattern_symmetric = true;
   free(irn);
   free(jcn);
   free(val);
@@ -2341,23 +2026,17 @@ SparseMatrix SparseMatrix_from_dense(int m, int n, double *x){
 
 }
 
-
-int SparseMatrix_distance_matrix(SparseMatrix D0, int weighted, double **dist0){
+void SparseMatrix_distance_matrix(SparseMatrix D0, double **dist0) {
   /*
     Input:
-    D: the graph. If weighted, the entry values is used.
-    weighted: whether to treat the graph as weighted
+    D: the graph. Entry values are unused.
     Output:
     dist: of dimension nxn, dist[i*n+j] gives the distance of node i to j.
-    return: flag. if not zero, the graph is not connected, or out of memory.
   */
   SparseMatrix D = D0;
   int m = D->m, n = D->n;
   int *levelset_ptr = NULL, *levelset = NULL, *mask = NULL;
-  double *dist = NULL;
-  int nlist, *list = NULL;
-  int flag = 0, i, j, k, nlevel;
-  double dmax;
+  int i, j, k, nlevel;
 
   if (!SparseMatrix_is_symmetric(D, false)){
     D = SparseMatrix_symmetrize(D, false);
@@ -2369,21 +2048,13 @@ int SparseMatrix_distance_matrix(SparseMatrix D0, int weighted, double **dist0){
   if (!(*dist0)) *dist0 = gv_calloc(n * n, sizeof(double));
   for (i = 0; i < n*n; i++) (*dist0)[i] = -1;
 
-  if (!weighted){
-    for (k = 0; k < n; k++){
-      SparseMatrix_level_sets(D, k, &nlevel, &levelset_ptr, &levelset, &mask, TRUE);
-      assert(levelset_ptr[nlevel] == n);
-      for (i = 0; i < nlevel; i++) {
-	for (j = levelset_ptr[i]; j < levelset_ptr[i+1]; j++){
-	  (*dist0)[k*n+levelset[j]] = i;
-	}
+  for (k = 0; k < n; k++) {
+    SparseMatrix_level_sets(D, k, &nlevel, &levelset_ptr, &levelset, &mask, true);
+    assert(levelset_ptr[nlevel] == n);
+    for (i = 0; i < nlevel; i++) {
+      for (j = levelset_ptr[i]; j < levelset_ptr[i+1]; j++) {
+        (*dist0)[k*n+levelset[j]] = i;
       }
-     }
- } else {
-    list = gv_calloc(n, sizeof(int));
-    for (k = 0; k < n; k++){
-      dist = &((*dist0)[k*n]);
-      flag = Dijkstra(D, k, dist, &nlist, list, &dmax);
     }
   }
 
@@ -2392,6 +2063,4 @@ int SparseMatrix_distance_matrix(SparseMatrix D0, int weighted, double **dist0){
   free(mask);
   
   if (D != D0) SparseMatrix_delete(D);
-  free(list);
-  return flag;
 }
