@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -11,6 +11,10 @@
 
 #include "config.h"
 #include "gdgen_text.h"
+#include <assert.h>
+#include <float.h>
+#include <limits.h>
+#include <stdbool.h>
 #include <stdlib.h>
 #include <stddef.h>
 #include <string.h>
@@ -25,11 +29,11 @@
 /* for gvcolor_t */
 #include <common/color.h>
 
-#include <cgraph/agxbuf.h>
-#include <cgraph/alloc.h>
 #include <cgraph/cgraph.h>
 #include <cgraph/strview.h>
 #include <common/render.h>
+#include <util/agxbuf.h>
+#include <util/alloc.h>
 
 /* for wind() */
 #include <pathplan/pathutil.h>
@@ -41,7 +45,7 @@ typedef enum { FORMAT_VRML, } format_type;
 typedef struct {
   double Scale;
   double MinZ;
-  int Saw_skycolor;
+  bool Saw_skycolor;
 
   gdImagePtr im;
   FILE *PNGfile;
@@ -149,23 +153,23 @@ static int color_index(gdImagePtr im, gvcolor_t color)
 static int set_penstyle(GVJ_t * job, gdImagePtr im, gdImagePtr brush)
 {
     obj_state_t *obj = job->obj;
-    int i, pen, pencolor, transparent, width, dashstyle[40];
+    int i, pen, pencolor, transparent, width, dashstyle[20];
 
     pen = pencolor = color_index(im, obj->pencolor);
     transparent = gdImageGetTransparent(im);
     if (obj->pen == PEN_DASHED) {
-        for (i = 0; i < 20; i++)
+        for (i = 0; i < 10; i++)
             dashstyle[i] = pencolor;
-        for (; i < 40; i++)
+        for (; i < 20; i++)
             dashstyle[i] = transparent;
         gdImageSetStyle(im, dashstyle, 20);
         pen = gdStyled;
     } else if (obj->pen == PEN_DOTTED) {
         for (i = 0; i < 2; i++)
             dashstyle[i] = pencolor;
-        for (; i < 24; i++)
+        for (; i < 12; i++)
             dashstyle[i] = transparent;
-        gdImageSetStyle(im, dashstyle, 24);
+        gdImageSetStyle(im, dashstyle, 12);
         pen = gdStyled;
     }
     width = obj->penwidth * job->scale.x;
@@ -173,7 +177,7 @@ static int set_penstyle(GVJ_t * job, gdImagePtr im, gdImagePtr brush)
         width = PENWIDTH_NORMAL;  /* gd can't do thin lines */
     gdImageSetThickness(im, width);
     /* use brush instead of Thickness to improve end butts */
-    if (width != PENWIDTH_NORMAL) {
+    if (width != (int)PENWIDTH_NORMAL) {
         brush = gdImageCreate(width, width);
         gdImagePaletteCopy(brush, im);
         gdImageFilledRectangle(brush, 0, 0, width - 1, width - 1, pencolor);
@@ -194,8 +198,8 @@ static void vrml_begin_page(GVJ_t *job)
     state->Scale = (double) DEFAULT_DPI / POINTS_PER_INCH;
     gvputs(job,   "#VRML V2.0 utf8\n");
 
-    state->Saw_skycolor = FALSE;
-    state->MinZ = MAXDOUBLE;
+    state->Saw_skycolor = false;
+    state->MinZ = DBL_MAX;
     gvputs(job,   "Group { children [\n"
                   "  Transform {\n");
     gvprintf(job, "    scale %.3f %.3f %.3f\n", .0278, .0278, .0278);
@@ -412,9 +416,7 @@ collinear (pointf * A)
  * Return true if bezier points are collinear
  * At present, just check with 4 points, the common case.
  */
-static int
-straight (pointf * A, int n)
-{
+static int straight(pointf *A, size_t n) {
     if (n != 4) return 0;
     return collinear(A) && collinear(A + 1);
 }
@@ -471,22 +473,21 @@ nearTail (GVJ_t* job, pointf a, Agedge_t* e)
     /* this is gruesome, but how else can we get z coord */
 #define GETZ(jp,op,p,e) (nearTail(jp,p,e)?op->tail_z:op->head_z) 
 
-static void
-vrml_bezier(GVJ_t *job, pointf *A, int n, int filled) {
+static void vrml_bezier(GVJ_t *job, pointf *A, size_t n, int filled) {
     (void)filled;
 
     obj_state_t *obj = job->obj;
     edge_t *e = obj->u.e;
     double fstz, sndz;
     pointf p1, V[4];
-    int i, j, step;
+    int step;
     state_t *state = job->context;
 
     assert(e);
 
     fstz = state->Fstz = obj->tail_z; 
     sndz = state->Sndz = obj->head_z;
-    if (straight(A,n)) {
+    if (straight(A, n)) {
 	doSegment (job, A, gvrender_ptf(job, ND_coord(agtail(e))),state->Fstz,gvrender_ptf(job, ND_coord(aghead(e))),state->Sndz);
 	return;
     }
@@ -494,12 +495,12 @@ vrml_bezier(GVJ_t *job, pointf *A, int n, int filled) {
     gvputs(job,   "Shape { geometry Extrusion  {\n"
                   "  spine [");
     V[3] = A[0];
-    for (i = 0; i + 3 < n; i += 3) {
+    for (size_t i = 0; i + 3 < n; i += 3) {
 	V[0] = V[3];
-	for (j = 1; j <= 3; j++)
+	for (size_t j = 1; j <= 3; j++)
 	    V[j] = A[i + j];
 	for (step = 0; step <= BEZIERSUBDIVISION; step++) {
-	    p1 = Bezier(V, 3, (double) step / BEZIERSUBDIVISION, NULL, NULL);
+	    p1 = Bezier(V, (double)step / BEZIERSUBDIVISION, NULL, NULL);
 	    gvprintf(job, " %.3f %.3f %.3f", p1.x, p1.y,
 		    interpolate_zcoord(job, p1, A[0], fstz, A[n - 1], sndz));
 	}
@@ -568,15 +569,14 @@ static void doArrowhead (GVJ_t *job, pointf * A)
                   "}\n");
 }
 
-static void vrml_polygon(GVJ_t *job, pointf * A, int np, int filled)
-{
+static void vrml_polygon(GVJ_t *job, pointf *A, size_t np, int filled) {
     obj_state_t *obj = job->obj;
     node_t *n;
     edge_t *e;
     double z = obj->z;
     pointf p, mp;
     gdPoint *points;
-    int i, pen;
+    int pen;
     gdImagePtr brush = NULL;
     double theta;
     state_t *state = job->context;
@@ -587,7 +587,7 @@ static void vrml_polygon(GVJ_t *job, pointf * A, int np, int filled)
 	    obj->fillcolor.u.rgba[0] / 255.,
 	    obj->fillcolor.u.rgba[1] / 255.,
 	    obj->fillcolor.u.rgba[2] / 255.);
-	state->Saw_skycolor = TRUE;
+	state->Saw_skycolor = true;
 	break;
     case CLUSTER_OBJTYPE:
 	break;
@@ -595,14 +595,15 @@ static void vrml_polygon(GVJ_t *job, pointf * A, int np, int filled)
 	n = obj->u.n;
 	pen = set_penstyle(job, state->im, brush);
 	points = gv_calloc(np, sizeof(gdPoint));
-	for (i = 0; i < np; i++) {
+	for (size_t i = 0; i < np; i++) {
 	    mp = vrml_node_point(job, n, A[i]);
 	    points[i].x = ROUND(mp.x);
 	    points[i].y = ROUND(mp.y);
 	}
+	assert(np <= INT_MAX);
 	if (filled)
-	    gdImageFilledPolygon(state->im, points, np, color_index(state->im, obj->fillcolor));
-	gdImagePolygon(state->im, points, np, pen);
+	    gdImageFilledPolygon(state->im, points, (int)np, color_index(state->im, obj->fillcolor));
+	gdImagePolygon(state->im, points, (int)np, pen);
 	free(points);
 	if (brush)
 	    gdImageDestroy(brush);
@@ -617,7 +618,7 @@ static void vrml_polygon(GVJ_t *job, pointf * A, int np, int filled)
 	gvputs(job,   "  }\n"
 	              "  geometry Extrusion {\n"
 	              "    crossSection [");
-	for (i = 0; i < np; i++) {
+	for (size_t i = 0; i < np; i++) {
 	    p.x = A[i].x - ND_coord(n).x;
 	    p.y = A[i].y - ND_coord(n).y;
 	    gvprintf(job, " %.3f %.3f,", p.x, p.y);
@@ -637,7 +638,7 @@ static void vrml_polygon(GVJ_t *job, pointf * A, int np, int filled)
 	    static int flag;
 	    if (!flag) {
 		flag++;
-		agerr(AGWARN,
+		agwarningf(
 		  "vrml_polygon: non-triangle arrowheads not supported - ignoring\n");
 	    }
 	}
@@ -646,12 +647,12 @@ static void vrml_polygon(GVJ_t *job, pointf * A, int np, int filled)
 	    return;
 	}
 	p.x = p.y = 0.0;
-	for (i = 0; i < np; i++) {
+	for (size_t i = 0; i < np; i++) {
 	    p.x += A[i].x;
 	    p.y += A[i].y;
 	}
-	p.x = p.x / np;
-	p.y = p.y / np;
+	p.x /= (int)np;
+	p.y /= (int)np;
 
 	/* it is bad to know that A[1] is the aiming point, but we do */
 	theta =
